import typing


class UserConfig:
    name: str
    friendly_name: str
    type: typing.Type
    callback: typing.Optional[typing.Callable]
    redact_current_value: bool = False

    def __init__(
        self,
        name: str,
        friendly_name: str,
        callback: typing.Optional[typing.Callable] = None,
        redact_current_value: bool = False,
    ):
        self.name = name
        self.friendly_name = friendly_name
        self.callback = callback
        self.redact_current_value = redact_current_value

    @staticmethod
    def split(value: str, separator=",") -> typing.List[str]:
        return list(filter(None, value.split(separator)))



import argparse

from reptor.lib.plugins.UploadBase import UploadBase


class DeleteProjects(UploadBase):
    meta = {
        "name": "DeleteProjects",
        "summary": "Deletes projects by title",
    }

    def __init__(self, **kwargs):
        super().__init__(**kwargs)
        self.project_id = kwargs.get("project_id")
        self.title_contains = kwargs.get("title_contains") or ""
        self.exclude_title_contains = kwargs.get("exclude_title_contains") or ""
        self.no_dry_run = kwargs.get("no_dry_run")

    @classmethod
    def add_arguments(cls, parser, plugin_filepath=None):
        super().add_arguments(parser, plugin_filepath=plugin_filepath)
        parser.add_argument(
            "--title-contains",
            metavar="SEARCHTERM",
            action="store",
            dest="title_contains",
            help="Match string in title",
        )
        parser.add_argument(
            "--titlecontains",
            metavar="SEARCHTERM",
            action="store",
            dest="title_contains",
            help=argparse.SUPPRESS,
        )
        parser.add_argument(
            "--exclude-title-contains",
            metavar="SEARCHTERM",
            action="store",
            dest="exclude_title_contains",
            help="Matched strings in title are not deleted",
        )
        parser.add_argument(
            "--excludetitlecontains",
            "--exclude-titlecontains",
            "--excludetitle-contains",
            metavar="SEARCHTERM",
            action="store",
            dest="exclude_title_contains",
            help=argparse.SUPPRESS,
        )
        parser.add_argument(
            "--no-dry-run",
            dest="no_dry_run",
            help="Do delete projects, default is dry-run",
            action="store_true",
        )
        parser.add_argument(
            "--nodryrun",
            "--nodry-run",
            "--nodry-run",
            dest="no_dry_run",
            help=argparse.SUPPRESS,
            action="store_true",
        )

    def _search_and_delete(self):
        projects = self.reptor.api.projects.search(search_term=self.title_contains)
        matched = False
        for project in projects:
            if self.title_contains.lower() in project.name.lower():
                if (
                    self.exclude_title_contains
                    and self.exclude_title_contains.lower()
                    in project.name.lower()
                ):
                    continue
                matched = True
                if not self.no_dry_run:
                    self.display(f'Would delete project "{project.name}"')
                else:
                    self.reptor.api.projects.delete_project(project_id=project.id)
                    self.display(f"Deleted project {project.name}")

        if not matched:
            self.display("No projects matched.")

    def run(self):
        if self.title_contains or self.exclude_title_contains:
            self._search_and_delete()
        elif self.project_id:
            if not self.no_dry_run:
                self.display(f'Would delete project with ID "{self.project_id}"')
            else:
                self.reptor.api.projects.delete_project(project_id=self.project_id)
                self.display(f"Deleted project with ID {self.project_id}")

        if not self.no_dry_run:
            self.display('\nDry-run, delete with "--no-dry-run".')


loader = DeleteProjects



import os
import subprocess
from pathlib import Path

import pytest

from reptor.plugins.core.Conf.tests.conftest import get_note


@pytest.mark.integration
class TestIntegrationFile(object):
    def test_upload_file(self):
        archive_path = Path(os.path.dirname(__file__)) / "data/archive.7z"

        # Without filename
        p = subprocess.Popen(
            ["reptor", "file", archive_path],
            stdout=subprocess.PIPE,
            stderr=subprocess.PIPE,
        )
        out, err = p.communicate()
        assert p.returncode == 0

        note = get_note("Uploads", None)
        note_last_line = note["text"].splitlines()[-1]
        assert "[archive.7z]" in note_last_line

        # With filename
        fn = "myarchive.7z"
        p = subprocess.Popen(
            ["reptor", "file", "-fn", fn, archive_path],
            stdout=subprocess.PIPE,
        )
        p.communicate()
        assert p.returncode == 0

        note = get_note("Uploads", None)
        note_last_line = note["text"].splitlines()[-1]
        assert fn in note_last_line



import argparse

from reptor.lib.plugins.UploadBase import UploadBase



class DeleteFindings(UploadBase):
    meta = {
        "name": "DeleteFindings",
        "summary": "Deletes findings by title",
    }

    def __init__(self, **kwargs):
        super().__init__(**kwargs)
        self.title_contains = kwargs.get("title_contains") or ""
        self.exclude_title_contains = kwargs.get("exclude_title_contains") or ""
        self.no_dry_run = kwargs.get("no_dry_run")

    @classmethod
    def add_arguments(cls, parser, plugin_filepath=None):
        super().add_arguments(parser, plugin_filepath=plugin_filepath)
        parser.add_argument(
            "--title-contains",
            metavar="SEARCHTERM",
            action="store",
            dest="title_contains",
            help="Match string in title",
        )
        parser.add_argument(
            "--titlecontains",
            metavar="SEARCHTERM",
            action="store",
            dest="title_contains",
            help=argparse.SUPPRESS
        )
        parser.add_argument(
            "--exclude-title-contains",
            metavar="SEARCHTERM",
            action="store",
            dest="exclude_title_contains",
            help="Matched strings in title are not deleted",
        )
        parser.add_argument(
            "--excludetitlecontains",
            "--exclude-titlecontains",
            "--excludetitle-contains",
            metavar="SEARCHTERM",
            action="store",
            dest="exclude_title_contains",
            help=argparse.SUPPRESS,
        )
        parser.add_argument(
            "--no-dry-run",
            dest="no_dry_run",
            help="Do delete findings, default is dry-run",
            action="store_true",
        )
        parser.add_argument(
            "--nodryrun",
            "--no-dryrun",
            "--nodry-run",
            dest="no_dry_run",
            help=argparse.SUPPRESS,
            action="store_true",
        )

    def run(self):
        findings = self.reptor.api.projects.get_findings()
        matched = False
        for finding in findings:
            if self.title_contains.lower() in finding.data.title.lower():
                if (
                    self.exclude_title_contains
                    and self.exclude_title_contains.lower()
                    in finding.data.title.lower()
                ):
                    continue
                matched = True
                if not self.no_dry_run:
                    self.display(f'Would delete finding "{finding.data.title}"')
                else:
                    self.reptor.api.projects.delete_finding(finding.id)
                    self.display(f"Deleted finding {finding.data.title}")

        if not matched:
            self.display("No findings matched.")
        elif not self.no_dry_run:
            self.display('\nDry-run, delete with "--no-dry-run".')


loader = DeleteFindings



import os
import pathlib
import subprocess

import pytest
import yaml

from reptor.plugins.core.Conf.tests.conftest import (
    notes_api,
    projects_api,
    read_until,
    templates_api,
)


@pytest.mark.integration
class TestIntegrationBurp(object):
    def test_burp_note(self, notes_api):
        input_path = (
            pathlib.Path(os.path.dirname(__file__)) / "data/burp.xml"
        )

        p = subprocess.Popen(
            ["reptor", "burp", "--upload", "--include", "16777984", "--debug"],
            stdin=subprocess.PIPE,
            stdout=subprocess.PIPE,
        )
        p.communicate(input=input_path.read_bytes())
        assert p.returncode == 0

        for parent in ["1.1.1.1", "34.249.203.140"]:
            note = notes_api.get_note_by_title(
                "🔵 Strict transport security not enforced",
                parent_notetitle=parent,
            )
            note_lines = note.text.splitlines()
            lines = [
                "**Type (plugin ID):** 16777984  ",
            ]
            assert all([line in note_lines for line in lines])

    def test_nessus_findings(self, projects_api):
        nmap_output_path = (
            pathlib.Path(os.path.dirname(__file__)) / "data/burp.xml"
        )

        p = subprocess.Popen(
            ["reptor", "burp", "--push-findings"],
            stdin=subprocess.PIPE,
            stdout=subprocess.PIPE,
        )
        p.communicate(input=nmap_output_path.read_bytes())
        assert p.returncode == 0

        projects_api.get_findings()
        assert "Vulnerable JavaScript dependency" in [
            f.data.title for f in projects_api.get_findings()
        ]

    def test_conf(self):
        # Must be last test as it modifies the config file
        # Test interactive config
        p = subprocess.Popen(
            ["reptor", "burp", "--conf"],
            stdin=subprocess.PIPE,
            stdout=subprocess.PIPE,
            # stderr=subprocess.PIPE,
        )
        read_until(p.stdout)
        p.stdin.write(b"medium-high\n")  # type: ignore
        p.stdin.flush()  # type: ignore

        read_until(p.stdout)
        p.stdin.write(b"1234,9876\n")  # type: ignore
        p.stdin.flush()  # type: ignore

        read_until(p.stdout)
        p.stdin.write(b"4567,5678\n")  # type: ignore
        p.stdin.flush()  # type: ignore

        read_until(p.stdout)
        p.stdin.write(b"y\n")  # type: ignore
        p.stdin.flush()  # type: ignore
        p.wait(timeout=5)

        # Assert config file was created correctly
        with open(pathlib.Path.home() / ".sysreptor/config.yaml") as f:
            config = yaml.safe_load(f)["burp"]

        assert all(
            [s in ["medium", "high"] for s in config["severity_filter"]]
        )
        assert config["excluded_plugins"] == ["1234", "9876"]
        assert config["included_plugins"] == ["4567", "5678"]



import json
import os
import pathlib
import subprocess
import time

import pytest
import requests
import yaml

from reptor.api.NotesAPI import NotesAPI
from reptor.api.ProjectDesignsAPI import ProjectDesignsAPI
from reptor.api.ProjectsAPI import ProjectsAPI
from reptor.api.TemplatesAPI import TemplatesAPI
from reptor.lib.reptor import Reptor


@pytest.fixture(scope="session", autouse=True)
def setUp():
    # Rename config file
    filename_timestamp = int(time.time())
    try:
        os.rename(
            pathlib.Path.home() / ".sysreptor/config.yaml",
            pathlib.Path.home() / f".sysreptor/config.{filename_timestamp}.yaml",
        )
    except FileNotFoundError:
        pass

    # Set API Token and URL
    conf()

    # Get design ID
    reptor = Reptor()
    if os.environ.get("HTTPS_PROXY", "").startswith("http://"):
        reptor._config._raw_config["insecure"] = True
        reptor._config._raw_config["cli"] = {"insecure": True}
    project_designs = ProjectDesignsAPI(reptor=reptor).search()
    for design in project_designs:
        if "Demo Calzone" in design.name:
            project_design_id = design.id
            break
    else:
        raise ValueError("Demo Calzone project design not found")

    # Create project
    projects_api = ProjectsAPI(reptor=reptor)
    project = projects_api.create_project(
        "Integration Test Project", project_design_id, tags=["integration-test"]
    )

    # Also add project_id
    conf(project_id=project.id)

    yield

    # Delete Project
    conf(project_id=project.id)
    reptor = Reptor()
    if os.environ.get("HTTPS_PROXY", "").startswith("http://"):
        reptor._config._raw_config["insecure"] = True
        reptor._config._raw_config["cli"] = {"insecure": True}
    projects_api = ProjectsAPI(reptor=reptor)
    projects_api.delete_project()

    # Remove integration test config file
    try:
        os.remove(pathlib.Path.home() / ".sysreptor/config.yaml")
    except FileNotFoundError:
        pass
    # Restore config file
    try:
        os.rename(
            pathlib.Path.home() / f".sysreptor/config.{filename_timestamp}.yaml",
            pathlib.Path.home() / ".sysreptor/config.yaml",
        )
    except FileNotFoundError:
        pass


@pytest.fixture(scope="session")
def projects_api():
    reptor = Reptor()
    if os.environ.get("HTTPS_PROXY", "").startswith("http://"):
        reptor._config._raw_config["insecure"] = True
    return ProjectsAPI(reptor=reptor)


@pytest.fixture(scope="session")
def notes_api():
    reptor = Reptor()
    reptor._config._raw_config["cli"] = {"private_note": False}
    if os.environ.get("HTTPS_PROXY", "").startswith("http://"):
        reptor._config._raw_config["cli"]["insecure"] = True
    return NotesAPI(reptor=reptor)


@pytest.fixture(scope="session")
def private_notes_api():
    reptor = Reptor()
    reptor._config._raw_config["cli"] = {"private_note": True}
    if os.environ.get("HTTPS_PROXY", "").startswith("http://"):
        reptor._config._raw_config["cli"]["insecure"] = True
    return NotesAPI(reptor=reptor)


@pytest.fixture(scope="session")
def project_design_api():
    reptor = Reptor()
    reptor._config._raw_config["cli"] = {"private_note": False}
    if os.environ.get("HTTPS_PROXY", "").startswith("http://"):
        reptor._config._raw_config["cli"]["insecure"] = True
    return ProjectDesignsAPI(reptor=reptor)


@pytest.fixture(scope="session")
def templates_api():
    reptor = Reptor()
    reptor._config._raw_config["cli"] = {"private_note": False}
    if os.environ.get("HTTPS_PROXY", "").startswith("http://"):
        reptor._config._raw_config["cli"]["insecure"] = True
    return TemplatesAPI(reptor=reptor)


@pytest.fixture(scope="module")
def uploads_id(notes_api):
    notes_api.write_note("Create Note")
    uploads_note = get_note("Uploads", None)
    assert uploads_note is not None
    return uploads_note["id"]


@pytest.fixture(scope="module")
def private_uploads_id(private_notes_api):
    private_notes_api.write_note("Create Note")
    uploads_note = get_note("Uploads", None, private=True)
    assert uploads_note is not None
    return uploads_note["id"]


@pytest.fixture(autouse=True, scope="session")
def delete_notes(setUp, notes_api):
    yield

    # Delete all notes via notes_api
    for note in get_notes():
        try:
            notes_api.delete_note(note["id"])
        except requests.exceptions.HTTPError:
            pass
    # Assert notes are gone
    notes = get_notes()
    assert len(notes) == 0


@pytest.fixture(autouse=True, scope="session")
def delete_findings(setUp, projects_api):
    yield

    # Delete findings via projects_api
    for finding in projects_api.get_findings():
        projects_api.delete_finding(finding.id)
    # Assert findings are gone
    findings = projects_api.get_findings()
    assert len(findings) == 0


def get_notes(private=False):
    cmd = ["reptor", "note", "--list", "--json"]
    if os.environ.get("HTTPS_PROXY", "").startswith("http://"):
        cmd.append("--insecure")
    if private:
        cmd.append("--private-note")

    p = subprocess.Popen(
        cmd,
        stdout=subprocess.PIPE,
    )
    notes, _ = p.communicate()
    notes = json.loads(notes.decode())
    assert p.returncode == 0
    return notes


def get_note(name, parent, notes=None, private=False):
    if notes is None:
        notes = get_notes(private=private)
    if parent:
        pass
    for note in notes:
        if note["title"] == name and note["parent"] == parent:
            return note


def read_until(f, eof=": "):
    content = ""
    while char := f.read(1).decode():
        content += char
        if content.endswith(eof):
            break
    return content


def conf(project_id=""):
    # Test reptor conf and thereby setup config file
    p = subprocess.Popen(
        ["reptor", "conf"],
        stdin=subprocess.PIPE,
        stdout=subprocess.PIPE,
        # stderr=subprocess.PIPE,
    )
    read_until(p.stdout)
    p.stdin.write(os.environ.get("SYSREPTOR_SERVER", "").encode("utf-8") + b"\n")  # type: ignore
    p.stdin.flush()  # type: ignore

    read_until(p.stdout)
    p.stdin.write(os.environ.get("SYSREPTOR_API_TOKEN", "").encode("utf-8") + b"\n")  # type: ignore
    p.stdin.flush()  # type: ignore

    read_until(p.stdout)
    p.stdin.write(project_id.encode("utf-8") + b"\n")  # type: ignore
    p.stdin.flush()  # type: ignore

    read_until(p.stdout)
    p.stdin.write(b"y\n")  # type: ignore
    p.stdin.flush()  # type: ignore
    p.wait(timeout=5)

    # Assert config file was created correctly
    with open(pathlib.Path.home() / ".sysreptor/config.yaml") as f:
        config = yaml.safe_load(f)
    assert config["server"] == os.environ.get("SYSREPTOR_SERVER", "")
    assert config["token"] == os.environ.get("SYSREPTOR_API_TOKEN", "")
    if project_id:
        assert config["project_id"] == project_id
    else:
        assert config["project_id"] is None



import os
import subprocess
from pathlib import Path

import pytest

from reptor.models.Note import Note
from reptor.plugins.core.Conf.tests.conftest import notes_api  # noqa: F401


@pytest.mark.integration
class TestIntegrationZap(object):
    @pytest.fixture(autouse=True)
    def tearDown(self, notes_api):  # noqa: F811
        yield
        # Delete Zap notes (prevents interference between xml and json)
        note = notes_api.get_note_by_title("Zap")
        notes_api.delete_note(note.id)

    @pytest.mark.parametrize("format", ["xml", "json"])
    def test_notes(self, format, notes_api):  # noqa: F811
        input_path = Path(os.path.dirname(__file__)) / f"data/zap-report.{format}"

        p = subprocess.Popen(
            ["reptor", "zap", f"--{format}", "--upload"],
            stdin=subprocess.PIPE,
            stdout=subprocess.PIPE,
        )
        p.communicate(input=input_path.read_bytes())
        assert p.returncode == 0

        note = notes_api.get_note_by_title(
            "http://localhost (7)", parent_notetitle="Zap"
        )
        assert isinstance(note, Note)

        note = notes_api.get_note_by_title(
            "🔴 Cross Site Scripting (Reflected)",
            parent_notetitle="http://localhost (7)",
        )
        assert isinstance(note, Note)
        note_lines = note.text.splitlines()
        assert "| Risk | High (Medium) |" in note_lines

        note = notes_api.get_note_by_title(
            "https://localhost (5)", parent_notetitle="Zap"
        )
        assert isinstance(note, Note)
        note = notes_api.get_note_by_title(
            "🟠 Application Error Disclosure", parent_notetitle="https://localhost (5)"
        )
        assert isinstance(note, Note)
        note_lines = note.text.splitlines()
        assert "| Number of Affected Instances | 34 |" in note_lines



import json
import os
import subprocess

import pytest

from reptor.plugins.core.Conf.tests.conftest import (
    get_note,
    notes_api,  # noqa: F401
    projects_api,  # noqa: F401
    project_design_api,  # noqa: F401
)


@pytest.mark.integration
class TestIntegrationProject(object):
    def test_render_project(self, projects_api, project_design_api):  # noqa: F811
        available_project_designs = project_design_api.search()
        for design in available_project_designs:
            if projects_api.project.project_type != design.id:
                break
        else:
            raise ValueError("No other project design found")

        projects_len = len(projects_api.search())
        p = subprocess.Popen(
            [
                "reptor",
                "project",
                "--render",
                "--upload",
                "--design",
                design.id,
            ],
        )
        p.communicate()
        assert p.returncode == 0

        note = get_note("Uploads", None)
        note_last_line = note["text"].splitlines()[-1]  # type: ignore
        assert f"[{projects_api.project.name}.pdf]" in note_last_line

        # --design duplicates project; check if cleaned up
        assert projects_len == len(projects_api.search())

    def test_finish_project(self, projects_api):  # noqa: F811
        # Assert project is active
        assert projects_api.project.readonly is False

        # Finish project
        p = subprocess.Popen(
            ["reptor", "project", "--finish"],
        )
        p.communicate()
        assert p.returncode == 0
        del projects_api._project_dict
        assert projects_api._get_project().readonly is True

        # Reactivate project
        p = subprocess.Popen(
            ["reptor", "project", "--reactivate"],
        )
        p.communicate()
        assert p.returncode == 0
        del projects_api._project_dict
        assert projects_api._get_project().readonly is False

    def test_export_tar_gz(self):
        fname = "myproject.tar.gz"
        try:
            os.remove(fname)
        except OSError:
            pass
        assert not os.path.isfile(fname)

        p = subprocess.Popen(
            ["reptor", "project", "--export", "tar.gz", "-o", fname],
        )
        p.communicate()
        assert p.returncode == 0

        assert os.path.isfile(fname)
        assert os.path.getsize(fname) > 0
        try:
            os.remove(fname)  # Cleanup
        except OSError:
            pass

    def test_export_json(self):
        p = subprocess.Popen(
            ["reptor", "project", "--export", "json", "-o", "-"],
            stdout=subprocess.PIPE,
        )
        output, _ = p.communicate()

        assert p.returncode == 0
        json.loads(output)  # Test if we can load json



import os
from pathlib import Path
from unittest.mock import MagicMock

import pytest

from reptor.lib.plugins.TestCaseToolPlugin import TestCaseToolPlugin
from reptor.models.ProjectDesign import ProjectDesign
from reptor.settings import DEFAULT_PROJECT_DESIGN

from ..Burp import Burp


class TestBurp(TestCaseToolPlugin):
    templates_path = os.path.normpath(Path(os.path.dirname(__file__)) / "../templates")

    @pytest.fixture(autouse=True)
    def setUp(self) -> None:
        Burp.setup_class(
            Path(os.path.dirname(self.templates_path)), skip_user_plugins=True
        )
        self.burp = Burp()

    def _load_xml_data(self, filename):
        self.burp.input_format = "xml"
        filepath = os.path.join(os.path.dirname(__file__), f"./data/{filename}.xml")
        with open(filepath, "r") as f:
            self.burp.raw_input = f.read()

    def test_generate_findings_missing_var(self):
        self._load_xml_data("burp")
        self.burp.load = MagicMock(return_value=self.burp.raw_input.encode())  # type: ignore
        self.burp._project_design = ProjectDesign(DEFAULT_PROJECT_DESIGN)
        self.reptor.api.templates.search = MagicMock(return_value=[])
        self.burp.preprocess_for_template = MagicMock(
            return_value=[
                {
                    "affected_components": [
                        "https://ginandjuice.shop/a'a%5c'b%22c%3e%3f%3e%25%7d%7d%25%25%3ec%3c[[%3f$%7b%7b%25%7d%7dcake%5c/post "
                        "(34.249.203.140)",
                    ],
                    "confidence": "Certain",
                    "finding_templates": "5243392",
                    "host": [
                        {"#text": "https://ginandjuice.shop", "@ip": "34.249.203.140"},
                    ],
                    "issueBackground": "Issue Background",
                    "issueDetailItems": {"issueDetailItem": "Other: AWSALB"},
                    "location": ["/catalog"],
                    "name": "TLS cookie without secure flag set",
                    "path": ["/catalog"],
                    "references": [],
                    "remediationBackground": "Remediation Background",
                    "serialNumber": [
                        "5305597333036717056",
                        "7187414666400253952",
                        "2235271953881709568",
                        "7889728362258169856",
                    ],
                    "severity": "info",
                    "severity_score": 0,
                    "type": "5243392",
                    "vulnerabilityClassifications": "<ul>\n"
                    "<li><a "
                    'href="https://cwe.mitre.org/data/definitions/614.html">CWE-614: '
                    "Sensitive Cookie in HTTPS Session Without "
                    "'Secure' Attribute</a></li>\n"
                    "</ul>",
                }
            ]
        )
        type = "5243392"
        self.burp.included_plugins = {type}
        findings = self.burp.generate_findings()
        preprocessed = self.burp.preprocess_for_template()

        assert len(preprocessed) == len(findings)

        for finding in findings:
            assert finding.data.severity.value in self.burp.risk_mapping

    def test_parse_included_plugins(self):
        self._load_xml_data("burp")
        type = "5243392"
        self.burp.included_plugins = {type}
        self.burp.parse()
        assert len(self.burp.parsed_input) > 0
        assert all(i.get("type") == type for i in self.burp.parsed_input)

    def test_parse_excluded_plugins(self):
        self._load_xml_data("burp")
        type = "5243392"
        self.burp.excluded_plugins = {type}
        self.burp.parse()
        assert len(self.burp.parsed_input) > 0
        assert all(i.get("type") != type for i in self.burp.parsed_input)

    @pytest.mark.parametrize(
        ["severity_filter", "raises_value_error"],
        [
            ({"low"}, False),
            ({"low", "medium"}, False),
            ({"low", "medium", "high"}, False),
            ({"low", "medium", "high", "info"}, False),
            ({"low", "medium", "high", "info", "invalid"}, True),
        ],
    )
    def test_parse_severity_filter(self, severity_filter, raises_value_error):
        self._load_xml_data("burp")
        self.burp.severity_filter = severity_filter
        if raises_value_error:
            with pytest.raises(ValueError):
                self.burp.parse()
        else:
            self.burp.parse()
            assert len(self.burp.parsed_input) > 0
            assert all(
                i.get("severity") in severity_filter for i in self.burp.parsed_input
            )

    def test_preprocess_for_template(self):
        self._load_xml_data("burp")
        self.burp.parse()
        findings = self.burp.preprocess_for_template()

        types = [i.get("type") for i in findings]
        assert len(types) == len(findings)

        for finding in findings:
            assert "requestresponse" not in finding
            assert isinstance(finding.get("host", list()), list)

    def test_generate_findings(self):
        self._load_xml_data("burp")
        self.burp.load = MagicMock(return_value=self.burp.raw_input.encode())  # type: ignore
        self.burp._project_design = ProjectDesign(DEFAULT_PROJECT_DESIGN)
        self.reptor.api.templates.search = MagicMock(return_value=[])
        findings = self.burp.generate_findings()
        preprocessed = self.burp.preprocess_for_template()

        assert len(preprocessed) == len(findings)

        for finding in findings:
            assert finding.data.severity.value in self.burp.risk_mapping

    def test_aggregate_by_ip(self):
        self._load_xml_data("burp")
        self.burp.parse()
        all_findings = self.burp.parsed_input
        assert isinstance(all_findings, list)
        assert len(all_findings) > 0
        all_types = {i.get("type") for i in all_findings}
        aggregated_findings = self.burp.aggregate_by_ip()

        all_aggregated_types = set()
        for _, findings in aggregated_findings.items():
            types = [i.get("type") for i in findings]
            assert len(types) == len(findings)
            all_aggregated_types.update(types)

        assert len(aggregated_findings) > 1
        assert all_types == all_aggregated_types



import subprocess

import pytest


@pytest.mark.integration
class TestIntegrationTranslate(object):
    def test_dry_run(self):
        p = subprocess.Popen(
            ["reptor", "translate", "--dry-run", "--to", "DE", "--debug"],
            stderr=subprocess.PIPE,
        )
        _, output = p.communicate()
        output = output.decode()
        assert p.returncode == 0

        translated_characters = 0
        for line in output.splitlines():
            if line.startswith("Translated "):
                translated_characters = int(line.split()[1])
                break
        assert translated_characters > 1000

        # Run with skip-fields
        p = subprocess.Popen(
            [
                "reptor",
                "translate",
                "--dry-run",
                "--to",
                "DE",
                "--skip-fields",
                "executive_summary",
            ],
            stderr=subprocess.PIPE,
        )
        _, output = p.communicate()
        output = output.decode()
        assert p.returncode == 0

        less_translated_characters = 0
        for line in output.splitlines():
            if line.startswith("Translated "):
                less_translated_characters = int(line.split()[1])
                break
        assert translated_characters > less_translated_characters > 1000



import json
import typing
from contextlib import nullcontext

import tomli_w
import yaml

from reptor.lib.plugins.Base import Base
from reptor.utils.table import make_table


class Project(Base):
    """
    This plugin is used to interact with projects via the sysreptor API.
    """

    meta = {
        "name": "Project",
        "summary": "Work with projects",
    }

    def __init__(self, **kwargs):
        super().__init__(**kwargs)
        self.search: typing.Optional[str] = kwargs.get("search")
        self.export: typing.Optional[str] = kwargs.get("export")
        self.render: bool = kwargs.get("render", False)
        self.design: typing.Optional[str]
        self.design = kwargs.get("design") or getattr(self, "design", None)
        self.design = None if self.design == "-" else self.design
        self.upload: bool = kwargs.get("upload", False)
        self.duplicate: bool = kwargs.get("duplicate", False)
        self.finish: typing.Optional[bool] = kwargs.get("finish", None)
        self.output: typing.Optional[str] = kwargs.get("output")
        self.format: str = kwargs.get("format", "plain")

    @classmethod
    def add_arguments(cls, parser, plugin_filepath=None):
        super().add_arguments(parser, plugin_filepath=plugin_filepath)
        project_parser = parser.add_mutually_exclusive_group()
        # Mutually exclusive options
        project_parser.add_argument(
            "--search",
            metavar="SEARCHTERM",
            help="Search for term",
            action="store",
            default=None,
        )
        project_parser.add_argument(
            "--export",
            help="Export project",
            choices=["tar.gz", "json", "toml", "yaml"],
            type=str.lower,
            action="store",
            dest="export",
            default=None,
        )
        project_parser.add_argument(
            "--render",
            help="Render project",
            action="store_true",
            dest="render",
        )
        project_parser.add_argument(
            "--duplicate",
            help="Duplicate project",
            action="store_true",
            dest="duplicate",
        )
        finish_group = project_parser.add_mutually_exclusive_group()
        finish_group.add_argument(
            "--finish",
            help="Set project as finished",
            action="store_true",
            default=None,
            dest="finish",
        )
        finish_group.add_argument(
            "--reactivate",
            help="Reactivate a finished project",
            action="store_false",
            default=None,
            dest="finish",
        )

        # Additional options
        parser.add_argument(
            "-o",
            "--output",
            metavar="FILENAME",
            help="Filename for output",
            action="store",
            default=None,
        )
        parser.add_argument(
            "--design",
            metavar="DESIGN ID",
            help="Render project with alternative design",
            action="store",
            default=None,
        )
        parser.add_argument(
            "--upload",
            action="store_true",
            help="Used with --export or --render; uploads file to note",
            dest="upload",
        )
        parser.add_argument(
            "--json",
            help="Used with --search; output as json",
            action="store_const",
            dest="format",
            const="json",
            default="plain",
        )

    def _export_project(self, format="tar.gz", filename=None, upload=False):
        stdout = False
        default_filename = self.reptor.api.projects.project.name or "project"

        if format == "tar.gz":
            if filename == "-":
                stdout = True
                filename = None
            archive_content = self.reptor.api.projects.export()
            self.deliver_file(
                content=archive_content,
                filename=filename or default_filename + ".tar.gz",
                upload=upload,
                stdout=stdout,
            )
            return

        if filename == "-" or filename is None:
            stdout = True
            filename = None
        project = self.reptor.api.projects.project.to_dict()
        output = ""
        if format == "json":
            output = json.dumps(project, indent=2)
        elif format == "toml":
            output = tomli_w.dumps(project)
        elif format == "yaml":
            output = yaml.dump(project)
        else:
            raise ValueError(f"Unknown format: {format}")
        self.deliver_file(
            content=output.encode(),
            filename=filename or f"{default_filename}.{format}",
            upload=upload,
            stdout=stdout,
        )

    def _search_project(self):
        projects = self.reptor.api.projects.search(self.search or "")

        if self.format == "json":
            self.print(
                json.dumps([project.to_dict() for project in projects], indent=2)
            )
        else:
            table = make_table(["Title", "ID", "Archived"])
            for project in projects:
                archived = ""
                if project.readonly:
                    archived = "[red]Yes[/red]"
                table.add_row(project.name, project.id, archived)

            self.console.print(table)

    def _duplicate_project(self):
        duplicated_project = self.reptor.api.projects.duplicate_project()
        project_title = duplicated_project.name
        project_id = duplicated_project.id
        self.success(f"Duplicated to '{project_title}' ({project_id})")

    def _finish_project(self, finish: typing.Optional[bool] = None):
        if finish is None:
            return
        if self.finish:
            if self.reptor.api.projects.finish_project():
                self.success("Project finished")
            else:
                self.error("Couldn't finish project")
        else:
            if not self.reptor.api.projects.finish_project(unfinish=True):
                self.success("Project reactivated")
            else:
                self.error("Couldn't reactivate project")

    def _render_project(self, filename=None, upload=False):
        stdout = False
        default_filename = (self.reptor.api.projects.project.name or "report") + ".pdf"
        if filename == "-":
            stdout = True
            filename = None
        with self.reptor.api.projects.duplicate_and_cleanup() if self.design else nullcontext():
            # Update design
            if self.design:
                self.reptor.api.projects.update_project_design(self.design, force=True)
            pdf_content = self.reptor.api.projects.render()
        self.deliver_file(
            content=pdf_content,
            filename=filename or default_filename,
            upload=upload,
            stdout=stdout,
        )

    def run(self):
        if self.export:
            self._export_project(self.export, filename=self.output, upload=self.upload)
        elif self.render:
            self._render_project(filename=self.output, upload=self.upload)
        elif self.duplicate:
            self._duplicate_project()
        elif self.finish is not None:
            self._finish_project(finish=self.finish)
        else:
            self._search_project()


loader = Project



import json
import os
from pathlib import Path
from unittest.mock import MagicMock, Mock

import pytest

from reptor.lib.plugins.TestCaseToolPlugin import TestCaseToolPlugin
from reptor.models.Finding import FindingRaw
from reptor.models.ProjectDesign import ProjectDesign
from reptor.settings import DEFAULT_PROJECT_DESIGN

from ..Sslyze import Sslyze


class TestSslyze(TestCaseToolPlugin):
    templates_path = os.path.normpath(Path(os.path.dirname(__file__)) / "../templates")
    context = {
        "data": [
            {
                "hostname": "example.com",
                "port": "443",
                "ip_address": "127.0.0.1",
                "vulnerabilities": {"heartbleed": True},
                "protocols": {
                    "sslv2": {"a": "b"},
                },
                "certinfo": {
                    "certificate_matches_hostname": False,
                    "certificate_untrusted": ["Android", "Apple"],
                },
                "misconfigurations": {
                    "compression": False,
                    "downgrade": True,
                    "no_secure_renegotiation": True,
                    "accepts_client_renegotiation": False,
                },
                "has_vulnerabilities": True,
                "flag_for_finding": True,
                "has_insecure_protocols": True,
                "has_weak_ciphers": True,
                "has_weak_protocols": True,
                "has_insecure_ciphers": True,
                "has_cert_issues": True,
                "has_misconfigurations": True,
            },
            {
                "hostname": "www.example.com",
                "port": "443",
                "ip_address": "127.0.0.1",
                "vulnerabilities": {"robot": True},
                "protocols": {
                    "sslv3": {"a": "b"},
                    "tlsv1_1": {"weak_ciphers": []},
                },
                "certinfo": {
                    "certificate_matches_hostname": True,
                    "certificate_untrusted": ["Android"],
                },
                "misconfigurations": {
                    "compression": True,
                    "downgrade": True,
                    "no_secure_renegotiation": True,
                    "accepts_client_renegotiation": True,
                },
                "has_vulnerabilities": True,
                "flag_for_finding": True,
                "has_insecure_protocols": True,
                "has_weak_ciphers": True,
                "has_weak_protocols": True,
                "has_insecure_ciphers": False,
                "has_cert_issues": True,
                "has_misconfigurations": True,
            },
            {
                "hostname": "ftp.example.com",
                "port": "443",
                "ip_address": "127.0.0.1",
                "vulnerabilities": {"openssl_ccs": True},
                "protocols": {"tlsv1_2": {"weak_ciphers": []}},
                "misconfigurations": {
                    "compression": False,
                    "downgrade": False,
                    "no_secure_renegotiation": False,
                    "accepts_client_renegotiation": False,
                },
                "certinfo": {
                    "certificate_matches_hostname": False,
                    "certificate_untrusted": [],
                },
                "has_vulnerabilities": True,
                "flag_for_finding": True,
                "has_insecure_protocols": True,
                "has_insecure_ciphers": True,
                "has_weak_ciphers": True,
                "has_weak_protocols": True,
                "has_cert_issues": True,
                "has_misconfigurations": False,
            },
        ],
    }

    @pytest.fixture(autouse=True)
    def setUp(self) -> None:
        Sslyze.setup_class(
            Path(os.path.dirname(self.templates_path)), skip_user_plugins=True
        )
        self.sslyze = Sslyze()

    def _load_json_data(self, filename):
        self.sslyze.input_format = "json"
        filepath = os.path.join(os.path.dirname(__file__), f"./data/{filename}.json")
        with open(filepath, "r") as f:
            self.sslyze.raw_input = f.read()

    def test_finding_content(self):
        # Patch API
        self.reptor.api.templates.search = Mock(return_value=[])
        self.sslyze._project_design = ProjectDesign(DEFAULT_PROJECT_DESIGN)

        self._load_json_data("sslyze_v5")
        self.sslyze.parsed_input = None
        self.sslyze.parse()
        data = self.sslyze.preprocess_for_template()
        assert isinstance(data, dict)
        data.update(self.context)

        # Patch preprocess_for_template
        self.sslyze.preprocess_for_template = Mock(return_value=data)
        self.sslyze.generate_findings()
        assert len(self.sslyze.findings) == 1

        summary = "We found that example.com:443 and 2 other services had a weak TLS setup. This might impact the confidentiality and integrity of your data in transit."

        assert summary in self.sslyze.findings[0].data.summary.value

        certinfo_data = """The certificates of example.com:443 and 2 other services are untrusted by common browsers:

* example.com:443 (unmatching hostname; untrusted by Android, Apple)
* www.example.com:443 (untrusted by Android)
* ftp.example.com:443 (unmatching hostname)"""
        assert certinfo_data in self.sslyze.findings[0].data.description.value

        misconfigurations_data = """Additionally, we detected the following misconfigurations:

|  | TLS Compression | Downgrade (no SCSV fallback) | No Secure Renegotiation | Client Renegotiation |
| ------- | ------- | ------- | ------- | ------- |
| example.com:443 |  <span style="color: green">No</span> | <span style="color: red">Yes</span> | <span style="color: red">Yes</span> | <span style="color: green">No</span> |
| www.example.com:443 |  <span style="color: red">Yes</span> | <span style="color: red">Yes</span> | <span style="color: red">Yes</span> | <span style="color: red">Yes</span> |
| ftp.example.com:443 |  <span style="color: green">No</span> | <span style="color: green">No</span> | <span style="color: green">No</span> | <span style="color: green">No</span> |"""
        assert misconfigurations_data in self.sslyze.findings[0].data.description.value

        vulnerabilities_data = """example.com:443 and 2 other services used outdated TLS libraries. This results in the following vulnerabilities:


|  | Heartbleed | Robot Attack | OpenSSL CCS |
| ------- | ------- | ------- | ------- |
| example.com:443 |  <span style="color: red">Vulnerable</span> | <span style="color: green">OK</span> | <span style="color: green">OK</span> |
| www.example.com:443 |  <span style="color: green">OK</span> | <span style="color: red">Vulnerable</span> | <span style="color: green">OK</span> |
| ftp.example.com:443 |  <span style="color: green">OK</span> | <span style="color: green">OK</span> | <span style="color: red">Vulnerable</span> |"""
        assert vulnerabilities_data in self.sslyze.findings[0].data.description.value

        protocols_data = """We also found out that example.com:443 and 2 other services had insecure ciphers or protocols enabled:

|  | SSLv2 | SSLv3 | TLS 1.0 | TLS 1.1 | Weak Ciphers | Insecure Ciphers |
| ------- | ------- | ------- | ------- | ------- | ------- | ------- |
| example.com:443 |  <span style="color: red">Yes</span> | <span style="color: green">No</span> | <span style="color: green">No</span> | <span style="color: green">No</span> | <span style="color: orange">Yes</span> | <span style="color: red">Yes</span> |
| www.example.com:443 |  <span style="color: green">No</span> | <span style="color: red">Yes</span> | <span style="color: green">No</span> | <span style="color: orange">Yes</span> | <span style="color: orange">Yes</span> | <span style="color: green">No</span> |
| ftp.example.com:443 |  <span style="color: green">No</span> | <span style="color: green">No</span> | <span style="color: green">No</span> | <span style="color: green">No</span> | <span style="color: orange">Yes</span> | <span style="color: red">Yes</span> |"""
        assert protocols_data in self.sslyze.findings[0].data.description.value

    def test_generate_and_push_findings(self):
        # Patch API
        self.reptor.api.templates.search = Mock(return_value=[])
        self.sslyze._project_design = ProjectDesign(DEFAULT_PROJECT_DESIGN)

        self._load_json_data("sslyze_v5")
        # Assert "create_finding" is called if no findings exist
        self.sslyze.reptor.api.projects.get_findings = Mock(return_value=[])
        self.reptor.api.projects.create_finding = MagicMock()

        self.sslyze.generate_and_push_findings()
        assert self.reptor.api.projects.create_finding.called

        # Assert "create_finding" is not called if finding with same title exists
        finding_raw = FindingRaw(
            {"data": {"title": "Weak TLS setup might impact encryption"}}
        )
        self.reptor.api.projects._project_dict = {"project_type": ""}
        self.reptor.api.project_designs.project_design = ProjectDesign()
        self.reptor.api.projects.get_findings = Mock(return_value=[finding_raw])
        self.reptor.api.projects.create_finding = MagicMock()

        self.sslyze.generate_and_push_findings()
        assert not self.sslyze.reptor.api.projects.create_finding.called

    def test_generate_findings(self):
        # Patch API
        self.reptor.api.projects.create_finding = MagicMock()
        self.reptor.api.projects.get_findings = Mock(return_value=[])
        self.sslyze.preprocess_for_template = Mock(return_value=self.context)
        self.sslyze._get_finding_from_remote_template = Mock(return_value=None)
        self.sslyze._project_design = ProjectDesign(DEFAULT_PROJECT_DESIGN)

        self._load_json_data("sslyze_v5")
        self.sslyze.generate_findings()
        assert len(self.sslyze.findings) == 1
        fndg = self.sslyze.findings[0]
        assert fndg.data.title.value == "Weak TLS setup might impact encryption"
        assert (
            "had insecure ciphers or protocols enabled" in fndg.data.description.value
        )
        assert "www.example.com:443 (127.0.0.1)" in [
            a.value for a in fndg.data.affected_components.value
        ]
        assert [r.value for r in fndg.data.references.value] == [
            "https://ssl-config.mozilla.org/",
            "https://ciphersuite.info/",
        ]

    def test_parse(self):
        result_dict = {"server_scan_results": ["b"]}
        self.sslyze.raw_input = json.dumps(result_dict)
        self.sslyze.parse()
        assert self.sslyze.parsed_input == result_dict

        self.sslyze.raw_input = [json.dumps(result_dict), json.dumps(result_dict)]
        self.sslyze.parse()
        assert len(self.sslyze.parsed_input.get("server_scan_results")) == 2

    def test_preprocess_for_template(self):
        result = {
            "data": [
                {
                    "hostname": "www.example.com",
                    "port": "443",
                    "ip_address": "127.0.0.1",
                    "protocols": {
                        "sslv2": {"weak_ciphers": [], "insecure_ciphers": []},
                        "tlsv1_2": {
                            "weak_ciphers": [
                                "DHE-RSA-AES256-SHA256",
                                "AES256-SHA256",
                                "ECDHE-RSA-AES256-SHA384",
                                "DHE-RSA-AES256-SHA",
                                "AES256-SHA",
                                "AES256-GCM-SHA384",
                                "DHE-RSA-AES128-SHA256",
                                "DHE-RSA-AES128-SHA",
                                "AES128-SHA",
                                "AES128-SHA256",
                                "AES128-GCM-SHA256",
                                "ECDHE-RSA-AES128-SHA256",
                            ],
                            "insecure_ciphers": [],
                        },
                    },
                    "has_weak_protocols": False,
                    "has_insecure_protocols": True,
                    "has_weak_ciphers": True,
                    "has_insecure_ciphers": False,
                    "certinfo": {
                        "certificate_matches_hostname": True,
                        "has_sha1_in_certificate_chain": False,
                        "certificate_untrusted": ["Android"],
                    },
                    "has_cert_issues": True,
                    "vulnerabilities": {
                        "heartbleed": True,
                        "openssl_ccs": True,
                        "robot": False,
                    },
                    "has_vulnerabilities": True,
                    "misconfigurations": {
                        "compression": False,
                        "downgrade": True,
                        "client_renegotiation": False,
                        "no_secure_renegotiation": True,
                    },
                    "has_misconfigurations": True,
                    "flag_for_finding": True,
                }
            ]
        }
        self._load_json_data("sslyze_v5")
        self.sslyze.parsed_input = None
        self.sslyze.parse()
        data = self.sslyze.preprocess_for_template()
        assert data == result

    def test_format(self):
        # result = """| Host | Port | Service | Version |"""
        self._load_json_data("sslyze_v5")

        # Protocols template
        protocols_result = """# Sslyze

## 🚩 www.example.com:443 (127.0.0.1)


**Protocols**

 * <span style="color: red">SSLv2</span>
 * <span style="color: green">TLS 1.2</span>


**Certificate Information**

 * <span style="color: red">Certificate untrusted by:</span>
    * Android
 * Certificate matches hostname: <span style="color: green">Yes</span>
 * SHA1 in certificate chain: <span style="color: green">No</span>


**Vulnerabilities**

 * Heartbleed: <span style="color: red">Yes</span>
 * Robot Attack: <span style="color: green">No</span>
 * OpenSSL CCS (CVE-2014-0224): <span style="color: red">Yes</span>


**Misconfigurations**

 * Compression: <span style="color: green">No</span>
 * Downgrade Attack (no SCSV fallback): <span style="color: red">Yes</span>
 * No Secure Renegotiation: <span style="color: red">Yes</span>
 * Client Renegotiation: <span style="color: green">No</span>


**Weak Cipher Suites**

 * <span style="color: orange">DHE-RSA-AES256-SHA256</span> (TLS 1.2)
 * <span style="color: orange">AES256-SHA256</span> (TLS 1.2)
 * <span style="color: orange">ECDHE-RSA-AES256-SHA384</span> (TLS 1.2)
 * <span style="color: orange">DHE-RSA-AES256-SHA</span> (TLS 1.2)
 * <span style="color: orange">AES256-SHA</span> (TLS 1.2)
 * <span style="color: orange">AES256-GCM-SHA384</span> (TLS 1.2)
 * <span style="color: orange">DHE-RSA-AES128-SHA256</span> (TLS 1.2)
 * <span style="color: orange">DHE-RSA-AES128-SHA</span> (TLS 1.2)
 * <span style="color: orange">AES128-SHA</span> (TLS 1.2)
 * <span style="color: orange">AES128-SHA256</span> (TLS 1.2)
 * <span style="color: orange">AES128-GCM-SHA256</span> (TLS 1.2)
 * <span style="color: orange">ECDHE-RSA-AES128-SHA256</span> (TLS 1.2)"""
        self.sslyze.format()
        assert self.sslyze.formatted_input.strip() == protocols_result.strip()

    def test_path_methods(self):
        plugin_path = Path(os.path.normpath(Path(os.path.dirname(__file__)) / ".."))
        template_paths = self.sslyze.get_plugin_dir_paths(
            plugin_path, "templates", skip_user_plugins=True
        )
        assert len(template_paths) == 1
        assert isinstance(template_paths[0], Path)
        assert template_paths[0] == plugin_path / "templates"

        templates = self.sslyze.get_filenames_from_paths(template_paths, "md")
        template_names = [
            "certinfo",
            "summary",
            "misconfigurations",
            "protocols",
            "vulnerabilities",
            "weak_ciphers",
        ]
        assert all([t in templates for t in template_names])



import argparse
import contextlib
import json
import sys
import typing

import tomli

from reptor.lib.plugins.UploadBase import UploadBase
from reptor.models.Section import Section as SectionModel
from reptor.models.Finding import Finding as FindingModel
from reptor.models.ProjectDesign import ProjectDesign


class PushProject(UploadBase):
    meta = {
        "name": "PushProject",
        "summary": "Push data to project from JSON or TOML",
    }

    def __init__(self, **kwargs):
        super().__init__(**kwargs)
        projectdata = kwargs.get("projectdata")
        if not isinstance(projectdata, dict):
            content = None
            if projectdata:
                content = projectdata.read().decode()
            content = self._read_input(content=content)
        else:
            content = projectdata

        self.projectdata: dict = content

    @classmethod
    def add_arguments(cls, parser, plugin_filepath=None):
        super().add_arguments(parser, plugin_filepath)
        parser.add_argument("projectdata", nargs="?", type=argparse.FileType("rb"))

    def run(self):
        len_sections = len(
            self.reptor.api.projects.update_report_fields(
                self.projectdata.get("report_data", {})
            )
        )
        if len_sections:
            self.log.success(
                f"Updated {len_sections} report section{'s'[:len_sections^1]}."
            )
        else:
            self.log.display("No report sections updated.")

        # Check for valid finding field data format
        project_design = self.reptor.api.project_designs.project_design
        findings = list()
        for finding in self.projectdata.get("findings", []):  # Check data format
            findings.append(
                (
                    finding,
                    FindingModel(
                        finding, project_design, strict_type_check=False
                    ),
                )
            )
        # Upload
        for finding, model in findings:
            self.reptor.api.projects.create_finding(finding)
            self.log.success(f'Created finding "{model.data.title}".')
        if not findings:
            self.log.display("No findings created.")

    def _read_input(self, content: typing.Optional[str] = None) -> dict:
        if content is None:
            # Read finding from stdin
            self.display("Reading from stdin...")
            content = sys.stdin.read()

        loaded_content: typing.Union[None, dict, list] = None
        with contextlib.suppress(json.JSONDecodeError):
            loaded_content = json.loads(content, strict=False)
        if not loaded_content:
            with contextlib.suppress(tomli.TOMLDecodeError):
                loaded_content = tomli.loads(content)
        if not loaded_content:
            raise ValueError("Could not decode stdin (expected JSON or TOML)")

        # Create model to assure compatibility of predefined fields
        assert isinstance(loaded_content, dict)
        report_data = loaded_content.get("report_data", {})
        assert isinstance(report_data, dict)
        SectionModel(
            {"data": report_data}, ProjectDesign(), strict_type_check=False
        )
        findings = loaded_content.get("findings", [])
        assert isinstance(findings, list)
        for finding in findings:
            assert isinstance(finding, dict)
            FindingModel(finding, ProjectDesign(), strict_type_check=False)
        return loaded_content


loader = PushProject



from unittest.mock import MagicMock

import pytest

from reptor.api.manager import APIManager
from reptor.lib.reptor import reptor
from reptor.models.Project import Project
from reptor.models.ProjectDesign import ProjectDesign

from ..CreateProject import CreateProject


class TestCreateProject:
    @pytest.fixture(autouse=True)
    def setUp(self):
        self.reptor = reptor
        self.reptor._config._raw_config["server"] = "https://demo.sysre.pt"
        self.reptor._config._raw_config["project_id"] = (
            "8a6ebd7b-637f-4f38-bfdd-3e8e9a24f64e"
        )
        self.reptor._api = APIManager(reptor=self.reptor)

    def get_mocked_object(self):
        create_project = CreateProject()
        create_project.design = "8a6ebd7b-1111-4f38-bfdd-3e8e9a24f64e"
        create_project.reptor.api.projects.create_project = MagicMock()
        create_project.reptor.api.projects.create_project.return_value = Project(
            {"id": "8a6ebd7b-637f-4f38-bfdd-3e8e9a24f64e"},
            ProjectDesign(),
        )
        self.reptor._config._write_to_file = MagicMock()
        self.reptor._config.load_config = MagicMock()
        self.reptor._config.load_config.return_value = {
            "server": "https://demo.sysre.pt",
            "project_id": "8a6ebd7b-637f-4f38-bfdd-3e8e9a24f64e",
        }
        return create_project

    def test_create_project(self):
        cp = self.get_mocked_object()
        create_project_mock = cp.reptor.api.projects.create_project
        cp.run()
        create_project_mock.assert_called_once_with(
            name="New Project created with Reptor",
            project_design="8a6ebd7b-1111-4f38-bfdd-3e8e9a24f64e",
            tags=["reptor"],
        )
        if cp.reptor._config._no_config_file:
            cp.reptor._config._write_to_file.assert_not_called()
        else:
            cp.reptor._config._write_to_file.assert_called_once_with(
                config={
                    "server": "https://demo.sysre.pt",
                    "project_id": "8a6ebd7b-637f-4f38-bfdd-3e8e9a24f64e",
                }
            )

    def test_create_project_no_update_config(self):
        cp = self.get_mocked_object()
        cp.no_update_config = True
        cp.run()
        cp.reptor.api.projects.create_project.assert_called_once_with(
            name="New Project created with Reptor",
            project_design="8a6ebd7b-1111-4f38-bfdd-3e8e9a24f64e",
            tags=["reptor"],
        )
        cp.reptor._config._write_to_file.assert_not_called()



import argparse
import glob
import json
import logging
import os
import sys
import typing
from functools import cached_property
from pathlib import Path
from xml.etree import ElementTree

import tomli
import xmltodict
from django.template import Context, Template
from django.template.loader import get_template

import reptor.settings as settings
from reptor.models.Finding import Finding
from reptor.models.FindingTemplate import FindingTemplate
from reptor.models.Note import NoteTemplate
from reptor.models.ProjectDesign import ProjectDesign
from reptor.utils.django_tags import custom_django_tags

from .Base import Base

log = logging.getLogger("reptor")


class ToolBase(Base):
    """The ToolBase provides plugin developers with all functionality that
    is commonly encountered when writing a plugin.


    Attributes:
        raw_input: Unformatted input, either from stdin or a file
        template: The .md file to be used during formatting
    """

    FINDING_PREFIX = "finding_"
    supports_multi_input = False

    def __init__(self, **kwargs):
        super().__init__(**kwargs)
        self.conf = kwargs.get("conf")
        self.action = kwargs.get("action")
        self.notetitle = self.notetitle or self.plugin_name
        self.push_findings = kwargs.get("push_findings")
        self.input = kwargs.get("input")
        if not isinstance(self.input, list):
            # Happens for tools that do not support multiple input files
            self.input = [self.input] if self.input else None
        self.note_icon = "🛠️"
        self.raw_input = None
        self.parsed_input = None
        self.formatted_input = None
        self.note_templates = None
        self.findings = []
        self.timestamp = (
            not self.reptor.get_config().get("cli", dict()).get("no_timestamp")
        )

        self.input_format = kwargs.get("format")
        if self.template_paths:
            settings.TEMPLATES[0]["DIRS"] = self.template_paths

    @classmethod
    def get_plugin_dir_paths(
        cls,
        plugin_path: Path,
        dirname: str,
        skip_user_plugins: bool = False,
    ) -> typing.List[Path]:
        if plugin_path is None:
            raise ValueError("plugin_path must not be None")
        dir_paths = list()
        if skip_user_plugins:
            user_plugin_path = None
        else:
            user_plugin_path = settings.PLUGIN_DIRS_USER / os.path.basename(plugin_path)

        for path in [
            user_plugin_path,
            plugin_path,
        ]:  # Keep order: files from user directory override
            if path is None:
                continue
            dir_path = Path(os.path.normpath(Path(path) / dirname))

            if dir_path not in dir_paths and os.path.isdir(dir_path):
                dir_paths.append(dir_path)

        # Return list of existing template paths
        return dir_paths

    @classmethod
    def get_filenames_from_paths(
        cls, dir_paths: typing.List[Path], filetype: str
    ) -> typing.List[str]:
        """
        takes a list of paths and returns a list of filenames without file extension
        """
        filetype = f"*.{filetype.strip('.')}"
        # Get template names from paths
        files_from_plugin_dir = list()
        for path in dir_paths:
            files = [
                os.path.basename(f).rsplit(".", 1)[0]
                for f in glob.glob(os.path.join(path, filetype))
            ]
            files_from_plugin_dir.extend(
                [t for t in files if t not in files_from_plugin_dir]
            )
        return sorted(files_from_plugin_dir)

    @classmethod
    def setup_class(cls, plugin_path: Path, skip_user_plugins: bool = False):
        # Get template and finding paths from plugin and userdir
        cls.finding_paths = cls.get_plugin_dir_paths(
            plugin_path,
            settings.FINDING_TEMPLATES_DIR_NAME,
            skip_user_plugins=skip_user_plugins,
        )
        cls.template_paths = cls.get_plugin_dir_paths(
            plugin_path,
            settings.PLUGIN_TEMPLATES_DIR_NAME,
            skip_user_plugins=skip_user_plugins,
        )

    @classmethod
    def add_arguments(cls, parser, plugin_filepath=None):
        super().add_arguments(parser, plugin_filepath=plugin_filepath)
        if plugin_filepath:
            cls.setup_class(Path(os.path.dirname(plugin_filepath)))

        parser.add_argument(
            "-i",
            "--input",
            action="store",
            dest="input",
            default=None,
            nargs="*" if cls.supports_multi_input else "?",
            help=f"Input file, if not stdin {'(multiple files allowed)' if cls.supports_multi_input else ''}",
        )

        action_group = parser.add_mutually_exclusive_group()
        action_group.title = "action_group"
        if cls.create_notes != ToolBase.create_notes:
            action_group.add_argument(
                "--format",
                action="store_const",
                dest="action",
                const="format",
                default="format",
            )
            action_group.add_argument(
                "--upload",
                action="store_const",
                dest="action",
                const="upload",
                default="format",
            )

        if cls._get_finding_methods():
            action_group.add_argument(
                "--push-findings",
                dest="push_findings",
                action="store_true",
            )
            action_group.add_argument(
                "--pushfindings",
                dest="push_findings",
                action="store_true",
                help=argparse.SUPPRESS,
            )
            action_group.add_argument(
                "--template-vars",
                action="store_const",
                dest="action",
                const="template-vars",
                help="Print template variables (needed for finding template customization).",
            )
            action_group.add_argument(
                "--templatevars",
                "--templatevariables",
                "--template-variables",
                action="store_const",
                dest="action",
                const="template-vars",
                help=argparse.SUPPRESS,
            )
        action_group.add_argument(
            "--parse",
            action="store_const",
            dest="action",
            const="parse",
            default="format",
        )
        if any(
            [
                cls.parse_xml != ToolBase.parse_xml,
                cls.parse_json != ToolBase.parse_json,
                cls.parse_csv != ToolBase.parse_csv,
            ]
        ):  # Prevent adding input_format_group if no parsing methods are implemented
            input_format_group = parser.add_mutually_exclusive_group()
            input_format_group.title = "input_format_group"

            # Add parsing options only if implemented by modules
            if cls.parse_xml != ToolBase.parse_xml:
                input_format_group.add_argument(
                    "--xml",
                    action="store_const",
                    dest="format",
                    const="xml",
                    default="raw",
                )
            if cls.parse_json != ToolBase.parse_json:
                input_format_group.add_argument(
                    "--json",
                    action="store_const",
                    dest="format",
                    const="json",
                    default="raw",
                )
            if cls.parse_csv != ToolBase.parse_csv:
                input_format_group.add_argument(
                    "--csv",
                    action="store_const",
                    dest="format",
                    const="csv",
                    default="raw",
                )
        if cls.get_filenames_from_paths(cls.finding_paths, "toml"):
            # If finding templates exist
            action_group.add_argument(
                "--upload-finding-templates",
                action="store_const",
                dest="action",
                const="upload-finding-templates",
                help="Upload local finding templates to SysReptor",
            )
            action_group.add_argument(
                "--uploadfindingtemplates",
                "--upload-findingtemplates",
                "--uploadfinding-templates",
                action="store_const",
                dest="action",
                const="upload-finding-templates",
                help=argparse.SUPPRESS,
            )

    @classmethod
    def get_input_format_group(cls, parser):
        # Find input_format_group
        for group in parser._mutually_exclusive_groups:
            if group.title == "input_format_group":
                return group
        else:
            raise ValueError("input_format_group not found")

    @classmethod
    def _get_finding_methods(cls) -> typing.List[typing.Callable]:
        finding_method_names = [
            func
            for func in dir(cls)
            if callable(getattr(cls, func)) and func.startswith(cls.FINDING_PREFIX)
        ]
        return [getattr(cls, n) for n in finding_method_names]

    def run(self):
        """
        The run method is always called by the main reptor application.

        The flow is:

        Parsing -> Formatting -> Uploading
        """
        if self.push_findings:
            self.generate_and_push_findings()
            if self.action == "upload":
                # Also upload to notes
                self.upload()
            return

        if self.action == "parse":
            self.parse()
            if self.parsed_input is not None:
                self.print(json.dumps(self.parsed_input, indent=2))
        elif self.action == "template-vars":
            self.parse()
            if p := self.preprocess_for_template():
                self.print(json.dumps(p, indent=2))
        elif self.action == "format":
            self.format()
            if self.formatted_input is not None:
                self.print(self.formatted_input)
        elif self.action == "upload":
            self.upload()
        elif self.action == "upload-finding-templates":
            self.upload_finding_templates()

    def load(self):
        """Puts the input into raw_input"""
        if self.input:
            self.raw_input = list()
            for filepath in self.input:
                with open(filepath, "r") as f:
                    self.raw_input.append(f.read())
            if len(self.raw_input) == 1:
                self.raw_input = self.raw_input[0]
                self.input = None
        else:
            self.display("Reading from stdin...")
            self.raw_input = sys.stdin.read()

    def parse_xml(self, as_dict=True):
        if as_dict:
            if isinstance(self.raw_input, list):
                self.parsed_input = list()
                for raw_input in self.raw_input:
                    self.parsed_input.append(xmltodict.parse(raw_input.strip()))
            else:
                self.parsed_input = xmltodict.parse(self.raw_input.strip())  # type: ignore
        else:
            if isinstance(self.raw_input, list):
                self.xml_root = list()
                for raw_input in self.raw_input:
                    self.xml_root.append(ElementTree.fromstring(raw_input))
            else:
                self.xml_root = ElementTree.fromstring(self.raw_input)

    def parse_json(self):
        if isinstance(self.raw_input, list):
            self.parsed_input = list()
            for raw_input in self.raw_input:
                self.parsed_input.append(json.loads(raw_input))
        else:
            self.parsed_input = json.loads(self.raw_input)  # type: ignore

    def parse_csv(self):
        raise NotImplementedError("Parse csv data is not implemented for this plugin.")

    def parse_raw(self):
        raise NotImplementedError("Parse raw data is not implemented for this plugin.")

    def parse(self):
        """
        Directs the input to the correct sub parsing method. For every toolbase
        plugin it is possible to handle --xml, --csv, --json or raw input.

        Depending on the arguments the corresponding sub parser method is called.

        If you decide not to support one of these, it won't be possible to provide
        the corresponding argument.
        """
        if not self.raw_input:
            self.load()

        if self.input_format == "xml":
            self.parse_xml()
        elif self.input_format == "json":
            self.parse_json()
        elif self.input_format == "csv":
            self.parse_csv()

    def format(self):
        """Checks if `self.parsed_input` is set.
        If not it starts the parsing process.
        Once there is parsed data it is run through the Django Templating Engine.
        """
        if not self.parsed_input:
            self.parse()

        if data := self.create_notes():
            if isinstance(data, NoteTemplate):
                data = [data]
            self.note_templates = data
            self.formatted_input = self.format_note_template(data)
            return
        self.log.warning(
            f"`create_notes` didn't return data for {self.plugin_name} plugin."
        )

    def format_note_template(self, note_templates: typing.List[NoteTemplate], level=1):
        formatted_input = ""
        for note_template in note_templates:
            if note_template.template:
                with custom_django_tags():
                    template = get_template(f"{note_template.template}.md")
                    note_template.text = Template(open(template.origin.name).read()).render(
                        Context(note_template.template_data, autoescape=False)
                    ) or ""
            if note_template.title:
                formatted_input += f"{'#' * level} {note_template.title}\n\n"
            if note_template.text:
                formatted_input += f"{note_template.text}\n\n"

            formatted_input += self.format_note_template(
                note_template.children, level=level + 1
            )
        return formatted_input

    def preprocess_for_template(self) -> dict:
        return {"data": self.parsed_input}

    def create_notes(
        self,
    ) -> typing.Union[NoteTemplate, typing.List[NoteTemplate], None]:
        raise NotImplementedError("Create notes is not implemented for this plugin.")

    def upload_finding_templates(self):
        finding_template_names = self.get_filenames_from_paths(
            self.finding_paths, "toml"
        )
        uploaded = 0
        for name in finding_template_names:
            if finding := self.get_local_finding_template(name):
                template_tag = f"{self.plugin_name}:{name}"
                remote_template = self.reptor.api.templates.get_templates_by_tag(
                    template_tag
                )
                if remote_template:
                    self.log.display(
                        f"Finding template with tag {template_tag} already exists. Skipping."
                    )
                    continue
                uploaded += 1
                finding["is_main"] = True
                finding = FindingTemplate(
                    {"translations": [finding], "tags": ["reptor", template_tag]}
                )
                self.reptor.api.templates.upload_template(finding)
                self.log.display(f'Uploaded finding template "{name}".')
        if uploaded:
            self.log.success(
                f"Successfully uploaded {uploaded} finding template{'s'[:uploaded^1]}."
            )

    def upload(self):
        """Uploads the `self.formatted_input` to sysreptor via the NotesAPI."""
        if not self.formatted_input:
            self.format()

        if self.note_templates:
            self.reptor.api.notes.write_note_templates(
                self.note_templates, timestamp=self.timestamp
            )
        else:
            parent_notetitle = "Uploads" if self.notetitle != "Uploads" else None
            self.reptor.api.notes.write_note(
                title=self.notetitle,
                text=self.formatted_input,
                parent_notetitle=parent_notetitle,
                icon_emoji=self.note_icon,
                timestamp=self.timestamp,
            )
        self.log.success("Successfully uploaded to notes.")

    def generate_and_push_findings(self) -> None:
        self.generate_findings()
        if len(self.findings) == 0:
            self.log.display("No findings generated.")
            return
        project_findings = [
            Finding(f, self._project_design)
            for f in self.reptor.api.projects.get_findings()
        ]
        project_finding_titles = [f.data.title.value for f in project_findings]
        for finding in self.findings:
            self.log.info(f'Checking if finding "{finding.data.title.value}" exists')
            if finding.data.title.value in project_finding_titles:
                self.log.display(
                    f'Finding "{finding.data.title.value}" already exists. Skipping.'
                )
                continue

            self.log.info(f'Pushing finding "{finding.data.title.value}"')
            if finding.template:
                # First create from template to keep template reference
                created_finding = self.reptor.api.projects.create_finding_from_template(
                    finding.template
                )
                # ...then update and add data
                self.reptor.api.projects.update_finding(
                    created_finding.id, finding.to_dict()
                )
            else:
                self.reptor.api.projects.create_finding(finding.to_dict())
            self.log.success(f'Pushed finding "{finding.data.title.value}"')

    @cached_property
    def _project_language(self) -> str:
        return self.reptor.api.projects.project.language

    @cached_property
    def _project_design(self) -> ProjectDesign:
        return self.reptor.api.project_designs.project_design

    def _get_finding_from_remote_template(
        self, template_tag: str
    ) -> typing.Optional[Finding]:
        """
        Returns first remote finding template with matching tag.
        """
        for finding_template in self.reptor.api.templates.get_templates_by_tag(
            template_tag
        ):
            # Take first matching template and fetch full data (search returns partial data only)
            self.log.info(f"Found remote finding template for {template_tag}.")
            finding_template = self.reptor.api.templates.get_template(
                finding_template.id
            )

            try:
                translation = [
                    t
                    for t in finding_template.translations
                    if t.language == self._project_language
                ][0]
            except IndexError:
                translation = [t for t in finding_template.translations if t.is_main][0]
                self.log.display(
                    f"No translation found for {self._project_language}. Taking main translation {translation.language}."
                )

            finding = Finding.from_translation(
                translation,
                project_design=self._project_design,
                strict_type_check=False,
            )
            finding.template = finding_template.id

            return finding

    def _get_finding_from_local_template(self, name: str) -> typing.Optional[Finding]:
        # Check if findings toml exists
        finding_dict = self.get_local_finding_template(name)
        if not finding_dict:
            return None

        try:
            finding = Finding(
                finding_dict,
                project_design=self._project_design,
                strict_type_check=False,
            )
        except ValueError:
            self.log.error("Finding data incompatible with project design.")
            return None
        return finding

    def generate_findings(self) -> typing.List[Finding]:
        """Generates findings from the parsed input.

        The findings are generated from the `self.parsed_input` and are
        written to the `self.findings` list
        """
        if not self.parsed_input:
            self.parse()
        self.findings = list()
        finding_methods = self._get_finding_methods()

        for method in finding_methods:
            finding = None
            finding_contexts = method(self)
            if finding_contexts is None:
                # Don't create finding if method returns None
                continue
            if not isinstance(finding_contexts, list):
                finding_contexts = [finding_contexts]

            for finding_context in finding_contexts:
                finding_names = list()
                if t := finding_context.get("finding_templates"):
                    if not isinstance(t, list):
                        t = [t]
                    finding_names.extend(t)
                t = method.__name__[len(self.FINDING_PREFIX) :]
                if t not in finding_names:
                    finding_names.append(t)

                for finding_name in finding_names:
                    # Check if remote finding template exists
                    template_tag = f"{self.plugin_name}:{finding_name}"
                    finding = self._get_finding_from_remote_template(template_tag)

                    # If not, check if local finding template exists
                    if not finding:
                        finding = self._get_finding_from_local_template(finding_name)
                    if finding:
                        break
                if not finding:
                    continue

                django_context = Context(finding_context, autoescape=False)
                for finding_data in finding.data:
                    # Iterate over all finding fields
                    if finding_data.value:
                        if isinstance(finding_data.value, str):
                            # Render template
                            with custom_django_tags():
                                finding_data.value = Template(finding_data.value).render(
                                    django_context
                                )
                            if finding_data.type == "number":
                                try:
                                    num_value = float(finding_data.value)
                                    if num_value.is_integer():
                                        finding_data.value = int(num_value)
                                    else:
                                        finding_data.value = num_value
                                except ValueError as e:
                                    # Conversion to number failed
                                    raise ValueError from e
                            elif finding_data.type == "boolean":
                                # Convert string to boolean
                                if finding_data.value.lower() in ["true", "1"]:
                                    finding_data.value = True
                                elif finding_data.value.lower() in ["false", "0"]:
                                    finding_data.value = False
                                else:
                                    raise ValueError(f'Invalid value "{finding_data.value}" for field "{finding_data.name}"')
                        
                    elif finding_data.type in [
                        "list",
                        "enum",
                        "combobox",
                        "date",
                        "number",
                        "boolean",
                    ]:
                        # A direct mapping of equal variable names from context to project design
                        # This is the only possibility to set lists.
                        if value := finding_context.get(finding_data.name):
                            try:
                                finding_data.value = value
                            except ValueError:
                                log.warning(
                                    f'Invalid value "{finding_data.value}" for field "{finding_data.name}"'
                                )
#
                self.findings.append(finding)
        return self.findings

    def get_local_finding_template(self, name: str) -> typing.Optional[dict]:
        """Loads a finding template from the local findings directory.

        Args:
            name: The name of the finding template to load

        Returns:
            A FindingRaw object or None if the template does not exist
        """
        if not self.finding_paths:
            self.debug("No path in finding_paths.")
            return None
        if not name.endswith(".toml"):
            name = f"{name}.toml"

        self.debug(
            f"Iterate through finding paths: {', '.join([str(p) for p in self.finding_paths])}"
        )
        for path in self.finding_paths:
            finding_template_path = path / name
            if os.path.isfile(finding_template_path):
                with open(finding_template_path, "rb") as f:
                    try:
                        finding_template = tomli.load(f)
                    except (tomli.TOMLDecodeError, TypeError):
                        self.log.warning(
                            f"Error while loading toml finding template {finding_template_path}."
                        )
                        continue
                return finding_template
            else:
                self.debug(f"No finding template found at {finding_template_path}.")
                continue
        return None

    @staticmethod
    def cvss2_to_3(cvss2: str) -> str:
        if cvss2.startswith("CVSS:3") or cvss2.startswith("CVSS:4"):
            return cvss2

        cvss2 = cvss2.replace("CVSS2#", "")
        cvss2_metrics = {
            k: v for k, v in (item.split(":") for item in cvss2.split("/"))
        }
        cvss3 = dict()
        # base
        cvss3["AV"] = cvss2_metrics.get("AV", "N")
        cvss3["AV"] = cvss3["AV"] if cvss3["AV"] in ["L", "A", "P", "N"] else "N"
        cvss3["AC"] = cvss2_metrics.get("AC", "L")
        cvss3["AC"] = cvss3["AC"] if cvss3["AC"] in ["H", "L"] else "L"
        auth_mapping = {
            "M": "H",
            "S": "L",
            "N": "N",
        }
        cvss3["PR"] = auth_mapping.get(cvss2_metrics.get("Au", "N"), "N")
        cvss3["UI"] = "N"
        cvss3["S"] = "U"
        impact_mapping = {
            "C": "H",
            "P": "L",
            "N": "N",
        }
        cvss3["C"] = impact_mapping.get(cvss2_metrics.get("C", "N"), "N")
        cvss3["I"] = impact_mapping.get(cvss2_metrics.get("I", "N"), "N")
        cvss3["A"] = impact_mapping.get(cvss2_metrics.get("A", "N"), "N")
        return f"CVSS:3.1/{'/'.join([f'{k}:{v}' for k, v in cvss3.items()])}"



import argparse
import io
import json
import tarfile
import uuid
from pathlib import Path

import tomli

from reptor.lib.plugins.Base import Base


def dir_or_file(path):
    p = Path(path)
    if not p.is_dir() and not p.is_file():
        raise argparse.ArgumentTypeError(f"readable_dir:{path} is not a valid path")
    return p


def build_tarinfo(name, size):
    info = tarfile.TarInfo(name=name)
    info.size = size
    return info


class PackArchive(Base):
    meta = {
        "name": "PackArchive",
        "summary": "Pack directories into a .tar.gz file",
    }

    def __init__(self, **kwargs):
        super().__init__(**kwargs)
        self.directories: list[Path] = kwargs.get("directories") or []
        self.output = kwargs.get("output")

    @classmethod
    def add_arguments(cls, parser, plugin_filepath=None):
        super().add_arguments(parser, plugin_filepath=plugin_filepath)

        parser.add_argument("directories", nargs="+", type=dir_or_file)
        parser.add_argument(
            "-o",
            "--output",
            type=argparse.FileType("wb"),
            default="packed_archive.tar.gz",
        )

    def load_file(self, path_input: Path):
        if not path_input.is_file():
            return None
        elif path_input.suffix == ".toml":
            data_dict = tomli.loads(path_input.read_text())
        else:
            data_dict = json.loads(path_input.read_text())
        if not isinstance(data_dict, dict) or not isinstance(data_dict.get('format'), str):
            return None
        return data_dict

    def add_to_archive(self, tar: tarfile.TarFile, path_input: Path, data_dict: dict, is_subresource=False):
        if not data_dict.get("id"):
            data_dict["id"] = str(uuid.uuid4())

        # Include file directories
        file_dirs = {}
        format = data_dict.get('format', '')
        if format.startswith("projects/"):
            file_dirs |= {
                f"{data_dict['id']}-images": f"{data_dict['id']}-images",
                f"{data_dict['id']}-files": f"{data_dict['id']}-files",
                f"{path_input.stem}-images": f"{data_dict['id']}-images",
                f"{path_input.stem}-files": f"{data_dict['id']}-files",
            }

            # Add project type
            project_type = data_dict.get("project_type")
            projecttype_path_input = path_input
            if not project_type:
                raise ValueError(f'Project type is missing in project: {path_input}')
            elif project_type.get('file'):
                projecttype_path_input = path_input.parent / project_type['file']
                project_type = self.load_file(projecttype_path_input)
                if not project_type:
                    raise ValueError(f'Invlaid reference to project type file: {projecttype_path_input}')
                data_dict['project_type'] = project_type
            self.add_to_archive(tar=tar, path_input=projecttype_path_input, data_dict=project_type, is_subresource=True)
        elif format.startswith("projecttypes/"):
            file_dirs |= {
                f"{data_dict['id']}-assets": f"{data_dict['id']}-assets",
                f"{path_input.stem}-assets": f"{data_dict['id']}-assets",
            }
        elif format.startswith("templates/"):
            file_dirs |= {
                f"{data_dict['id']}-images": f"{data_dict['id']}-images",
                f"{path_input.stem}-images": f"{data_dict['id']}-images",
            }
        elif format.startswith("notes/"):
            file_dirs |= {
                f"{data_dict['id']}-images": f"{data_dict['id']}-images",
                f"{data_dict['id']}-files": f"{data_dict['id']}-files",
                f"{path_input.stem}-images": f"{data_dict['id']}-images",
                f"{path_input.stem}-files": f"{data_dict['id']}-files",
            }

        # Add files to archive
        # Translate human-friendly names to archive names based on IDs
        for ds, dd in file_dirs.items():
            # Always add file list to data_dict
            data_key = dd.split("-")[-1]
            data_dict.setdefault(data_key, [])

            # Add directory contents to archive
            d_dir = Path(path_input).parent / ds
            if d_dir.is_dir():
                tar.add(d_dir, arcname=dd)
                for path_file in d_dir.glob("*"):
                    # Add file entry to data_dict
                    data_file = next(filter(lambda f: f.get('name') == path_file.name, data_dict.get(data_key, [])), None)
                    if not data_file:
                        data_file = {
                            'name': path_file.name,
                        }
                        data_dict[data_key].append(data_file)
                    if not data_file.get('id'):
                        data_file['id'] = str(uuid.uuid4())
                     
        # Add to archive
        if not is_subresource:
            data_json = json.dumps(data_dict, indent=2)
            tar.addfile(
                build_tarinfo(
                    name=data_dict["id"] + ".json", size=len(data_json)
                ),
                fileobj=io.BytesIO(data_json.encode()),
            )
        return True


    def run(self):
        if not self.directories:
            return
        with tarfile.open(fileobj=self.output, mode="w:gz") as tar:
            for path_dir in self.directories:
                if path_dir.is_dir():
                    # Add NOTICE files at top level
                    notice_path = path_dir / "NOTICE"
                    if notice_path.is_file():
                        notice_filename = "NOTICE"
                        if notice_filename in tar.getnames():
                            i = 0
                            while True:
                                i += 1
                                notice_filename = f"NOTICE ({str(i)})"
                                if notice_filename not in tar.getnames():
                                    break
                        tar.add(notice_path, arcname=notice_filename)

                    # Add files to archive
                    for path_input in list(path_dir.glob("*.toml")) + list(
                        path_dir.glob("*.json")
                    ):
                        data_dict = self.load_file(path_input)
                        if not data_dict:
                            continue
                        self.add_to_archive(tar=tar, path_input=path_input, data_dict=data_dict)
                elif path_dir.is_file():
                    data_dict = self.load_file(path_dir)
                    if not data_dict:
                        raise ValueError(f"Could not load file: {path_dir}")
                    self.add_to_archive(tar=tar, path_input=path_dir, data_dict=data_dict)

        self.success(f"Packed contents to {self.output.name}")


loader = PackArchive



from unittest.mock import MagicMock

import pytest
from requests.exceptions import HTTPError

from reptor.lib.reptor import reptor
from reptor.models.Note import Note

from ..NotesAPI import NotesAPI


class TestNotesAPI:
    class MockResponse:
        def __init__(self, content, status_code, raise_exception=False):
            self.content = content
            self.status_code = status_code
            self.raise_exception = raise_exception

        def raise_for_status(self):
            if self.raise_exception:
                raise HTTPError("Mocked HTTPError")
            return

        def json(self):
            return self.content

    test_note = {
        "id": "51abd3d3-803e-43a5-aa25-bf30b7fbf70a",
        "created": "2023-09-07T14:00:32.777492Z",
        "updated": "2023-09-07T14:00:45.409725Z",
        "title": "My Note",
        "text": "My Content",
        "checked": None,
        "icon_emoji": "\ud83d\udd25",
        "status_emoji": None,
        "order": 4,
        "parent": "8880ce39-90ed-4383-9320-d5d74b1ae34f",
    }

    @pytest.fixture(autouse=True)
    def setUp(self):
        reptor._config._raw_config["server"] = "https://demo.sysre.pt"
        reptor._config._raw_config["private_note"] = True
        self.notes = NotesAPI(reptor=reptor)

    def _mock_methods(self):
        self.notes.get_or_create_note_by_title = MagicMock(
            return_value=Note(self.test_note)
        )
        self.notes.create_note = MagicMock(return_value=Note(self.test_note))
        self.notes.put = MagicMock(return_value=self.MockResponse("", 201))

    def test_notes_api_init(self):
        # Test valid personal note
        reptor._config._raw_config["server"] = "https://demo.sysre.pt"
        reptor._config._raw_config["private_note"] = True
        try:
            n = NotesAPI(reptor=reptor)
            assert n.private_note
        except ValueError:
            self.fail("NotesAPI raised Error")

        # Test valid project note
        reptor._config._raw_config["server"] = "https://demo.sysre.pt"
        reptor._config._raw_config["private_note"] = False
        reptor._config._raw_config["project_id"] = (
            "2b5de38d-2932-4112-b0f7-42c4889dd64d"
        )
        try:
            n = NotesAPI(reptor=reptor)
            assert not n.private_note
        except ValueError:
            self.fail("NotesAPI raised Error")

        # Test missing project id and missing private_note
        reptor._config._raw_config["server"] = "https://demo.sysre.pt"
        reptor._config._raw_config["private_note"] = False
        reptor._config._raw_config["project_id"] = ""
        with pytest.raises(ValueError):
            NotesAPI(reptor=reptor)

        # Test missing server
        reptor._config._raw_config["server"] = ""
        reptor._config._raw_config["private_note"] = True
        with pytest.raises(ValueError):
            NotesAPI(reptor=reptor)



import os
from pathlib import Path
from unittest.mock import MagicMock

import pytest

from reptor.lib.plugins.TestCaseToolPlugin import TestCaseToolPlugin
from reptor.models.ProjectDesign import ProjectDesign
from reptor.settings import DEFAULT_PROJECT_DESIGN

from ..Nessus import Nessus


class TestNessus(TestCaseToolPlugin):
    templates_path = os.path.normpath(Path(os.path.dirname(__file__)) / "../templates")

    @pytest.fixture(autouse=True)
    def setUp(self) -> None:
        Nessus.setup_class(
            Path(os.path.dirname(self.templates_path)), skip_user_plugins=True
        )
        self.nessus = Nessus()

    def _load_xml_data(self, filename):
        self.nessus.input_format = "xml"
        filepath = os.path.join(os.path.dirname(__file__), f"./data/{filename}.xml")
        with open(filepath, "r") as f:
            self.nessus.raw_input = f.read()

    def test_aggregate_findings_with_severity_filter(self):
        self._load_xml_data("nessus_multi_host")
        self.nessus.severity_filter = {"medium"}
        self.nessus.parse()
        findings = self.nessus.aggregate_findings()
        assert len(findings) == 1
        assert len(findings['12218']["affected_components"]) == 2
        assert "192.168.1.112:5353 (mdns)" in findings["12218"]["affected_components"]
        assert "192.168.1.111:5353 (mdns)" in findings["12218"]["affected_components"]

    def test_parse_multi_input(self):
        self._load_xml_data("nessus_multi_host")
        self.nessus.parse()
        assert len(self.nessus.parsed_input) == 4
        self.nessus.raw_input = [self.nessus.raw_input, self.nessus.raw_input]
        self.nessus.parse()
        assert len(self.nessus.parsed_input) == 8

    def test_parse_multi_with_include(self):
        self._load_xml_data("nessus_multi_host")
        self.nessus.included_plugins = ["11219", "25216"]
        self.nessus.parse()
        p = self.nessus.parsed_input
        assert isinstance(p, list)
        assert len(p) == 2
        assert len(p[0]["findings"]) == 7
        assert len(p[1]["findings"]) == 2

    def test_parse_multi_with_exclude(self):
        self._load_xml_data("nessus_multi_host")
        self.nessus.excluded_plugins = ["11219", "25216"]
        self.nessus.parse()
        p = self.nessus.parsed_input
        assert isinstance(p, list)
        assert len(p) == 4
        assert len(p[0]["findings"]) == 5
        assert len(p[1]["findings"]) == 1

    def test_generate_findings(self):
        self._load_xml_data("nessus_multi_host")
        self.nessus.load = MagicMock(return_value=self.nessus.raw_input.encode())
        self.nessus._project_design = ProjectDesign(DEFAULT_PROJECT_DESIGN)
        self.reptor.api.templates.search = MagicMock(return_value=[])
        findings = self.nessus.generate_findings()
        assert len(findings) == 12
        assert (
            findings[0].data.title.value
            == "Samba NDR MS-RPC Request Heap-Based Remote Buffer Overflow"
        )
        assert (
            findings[11].data.title.value
            == "DNS Server hostname.bind Map Hostname Disclosure"
        )
        assert all(
            [
                f.data.cvss.value.startswith("CVSS:3.1")
                or f.data.cvss.value == ""
                or f.data.cvss.value == "n/a"
                for f in findings
            ]
        )
        assert all(
            [
                f.data.severity.value in ["info", "low", "medium", "high", "critical"]
                for f in findings
            ]
        )

        # Check correct affected componets
        assert [c.value for c in findings[0].data.affected_components.value] == [
            "10.15.10.11:443 (cifs)"
        ]
        assert [c.value for c in findings[1].data.affected_components.value] == [
            "10.15.10.11:443 (cifs)"
        ]
        assert [c.value for c in findings[2].data.affected_components.value] == [
            "10.15.10.11:53 (dns)"
        ]

        # Check plugin specific template (Service detection, plugin 22964)
        f = findings[8]
        assert f.data.title.value == "Detected network services"
        assert f.data.summary.value == "The remote service could be identified."
        assert f.data.cvss.value == "n/a"

    def test_preprocess_for_template(self):
        self._load_xml_data("nessus_multi_host")
        self.nessus.parse()
        p = self.nessus.preprocess_for_template()
        assert len(p) == 12
        assert len(set([f["plugin_name"] for f in p])) == 12
        assert all(isinstance(f.get("affected_components"), list) for f in p)
        assert p[0]["affected_components"] == ["10.15.10.11:443 (cifs)"]

        syn_finding = [f for f in p if f["pluginID"] == "11219"][0]
        affected_ips = set(
            [f.split(":")[0] for f in syn_finding["affected_components"]]
        )
        assert len(affected_ips) == 2

    def test_aggregate_findings(self):
        self._load_xml_data("nessus_multi_host")
        self.nessus.parse()
        findings = self.nessus.aggregate_findings()
        assert len(findings) == 12
        assert len(findings["11219"]["port"]) == 8
        assert len(findings["25216"]["port"]) == 1
        for finding in findings.values():
            assert (
                len(finding["port"])
                == len(finding["svc_name"])
                == len(finding["protocol"])
            )

    def test_parse_multi(self):
        self._load_xml_data("nessus_multi_host")
        self.nessus.parse()
        p = self.nessus.parsed_input
        assert isinstance(p, list)
        assert len(p) == 4
        assert p[0]["target"] == "10.15.10.11"
        assert len(p[0]["findings"]) == 12
        assert p[0]["findings"][0]["port"] == "443"
        assert p[0]["findings"][0]["svc_name"] == "cifs"
        assert isinstance(p[0]["findings"][3]["see_also"], list)
        assert len(p[0]["findings"][3]["see_also"]) == 1

    @pytest.mark.parametrize("filter", ["high-critical", "high,critical"])
    def test_parse_multi_with_severity_filter(self, filter):
        self._load_xml_data("nessus_multi_host")
        self.nessus.severity_filter = self.nessus._parse_severity_filter(filter)
        assert self.nessus.severity_filter == {"high", "critical"}
        self.nessus.parse()
        p = self.nessus.parsed_input
        assert isinstance(p, list)
        assert len(p) == 1  # Only one host with high/critical findings
        assert len(p[0]["findings"]) == 2  # Two high/critical findings

    def test_parse_single(self):
        self._load_xml_data("nessus_single_host")
        self.nessus.parse()
        p = self.nessus.parsed_input
        assert isinstance(p, list)
        assert len(p) == 1
        assert p[0]["target"] == "preprod.boardvantage.net"
        assert p[0]["host_ip"] == "12.233.108.201"
        assert len(p[0]["findings"]) == 6
        assert p[0]["findings"][0]["port"] == "0"



from collections import OrderedDict

SUBCOMMANDS_GROUPS = OrderedDict(
    {
        "core": ("Core", list()),
        "projects": ("Projects & Templates", list()),
        "uploads": ("Uploads", list()),
        "tools": ("Tools", list()),
        "importers": ("Importers", list()),
        "utils": ("Utils", list()),
        "other": ("Other", list()),
        "plugins": ("User Plugins", list()),
    }
)



import argparse
import typing
from urllib.parse import urlparse

from reptor.lib.plugins.ToolBase import ToolBase
from reptor.models.Note import NoteTemplate
from reptor.models.UserConfig import UserConfig


class Qualys(ToolBase):
    meta = {
        "author": "Syslifters",
        "name": "Qualys",
        "version": "1.0",
        "license": "MIT",
        "tags": [],
        "summary": "Qualys vulnerability importer",
    }

    score_mapping = {
        1: "info",
        2: "low",
        3: "medium",
        4: "high",
        5: "critical",
    }
    severity_mapping = {
        "info": "🟢",
        "low": "🔵",
        "medium": "🟡",
        "high": "🟠",
        "critical": "🔴",
    }
    supports_multi_input = True

    @property
    def user_config(self) -> typing.List[UserConfig]:
        return [
            UserConfig(
                name="severity_filter",
                friendly_name='Severity filter (e.g., "medium-critical")',
                callback=self._parse_severity_filter,
            ),
            UserConfig(
                name="excluded_plugins",
                friendly_name="Exclude plugin QIDs (comma-separated)",
                callback=UserConfig.split,
            ),
            UserConfig(
                name="included_plugins",
                friendly_name="Include plugin QIDs (comma-separated)",
                callback=UserConfig.split,
            ),
        ]

    def __init__(self, **kwargs):
        super().__init__(**kwargs)
        self.notetitle = kwargs.get("notetitle") or "Qualys"
        self.note_icon = "🛡️"
        self.input_format = "xml"
        self.timestamp = False
        self.severity_filter = getattr(
            self, "severity_filter", None
        ) or self._parse_severity_filter(kwargs.get("severity_filter", "info-critical"))

        self.included_plugins = getattr(self, "included_plugins", list())
        self.included_plugins += list(
            filter(None, ((kwargs.get("included_plugins")) or "").split(","))
        )
        self.excluded_plugins = getattr(self, "excluded_plugins", list())
        self.excluded_plugins += list(
            filter(None, ((kwargs.get("excluded_plugins")) or "").split(","))
        )

    def _parse_severity_filter(self, filter: str) -> set:
        if filter is None:
            return set(self.severity_mapping.keys())
        filter_elements = set()
        if "-" in filter:
            filter_elements = filter.split("-")
            try:
                filter_indexes = [
                    list(self.severity_mapping.keys()).index(f) for f in filter_elements
                ]
            except ValueError:
                raise ValueError(
                    "Invalid filter range. Use keywords from 'info,low,medium,high,critical'"
                )
            filter_elements = list()
            for i in range(min(filter_indexes), max(filter_indexes) + 1):
                filter_elements += [list(self.severity_mapping.keys())[i]]
        elif filter:
            filter_elements = filter.split(",")
            if not all(f in self.severity_mapping for f in filter_elements):
                raise ValueError(
                    "Invalid filter. Use keywords from 'info,low,medium,high,critical'"
                )
        if filter_elements:
            return set(filter_elements)
        else:
            return set(self.severity_mapping.keys())

    @classmethod
    def add_arguments(cls, parser, plugin_filepath=None):
        super().add_arguments(parser, plugin_filepath=plugin_filepath)
        parser.add_argument(
            "--severity-filter",
            help='Filter findings by severity comma-separated ("high,medium") or as range ("medium-critical")',
            action="store",
            dest="severity_filter",
            default=None,
        )
        parser.add_argument(
            "--severityfilter",
            help=argparse.SUPPRESS,
            action="store",
            dest="severity_filter",
            default=None,
        )
        parser.add_argument(
            "--exclude",
            help="Exclude plugin IDs, comma-separated",
            action="store",
            dest="excluded_plugins",
            default=None,
        )
        parser.add_argument(
            "--include",
            help="Include plugin IDs, comma-separated; default: all are included",
            action="store",
            dest="included_plugins",
            default=None,
        )

    def create_notes(self):
        self.parse()
        data = self.aggregate_by_target()
        # Create note structure
        ## Main note
        main_note = NoteTemplate()
        main_note.title = self.notetitle
        main_note.icon_emoji = self.note_icon

        ## Subnotes per target
        for findings in data:
            host_overview = NoteTemplate()
            host_overview.title = f"{findings[0]['target']}"
            host_overview.checked = False
            host_overview.template = "host-overview"
            host_overview.template_data = {"data": findings}
            for f in findings:
                finding_note = NoteTemplate()
                finding_note.title = f"{ self.severity_mapping.get(f.get('severity_label', 'info').lower()) } {f['TITLE']}"
                finding_note.checked = False
                finding_note.template = "finding"
                finding_note.template_data = f
                finding_note.force_new = True
                host_overview.children.append(finding_note)
            main_note.children.append(host_overview)
        if not main_note.children:
            return None
        return main_note

    def _get_findings_was(self, input) -> list:
        """
        Get findings from Web Application Scan report.
        """
        vulnerabilities = (
            input.get("WAS_SCAN_REPORT", dict())
            .get("RESULTS", dict())
            .get("VULNERABILITY_LIST")
            .get("VULNERABILITY", list())
        )
        glossary = (
            input.get("WAS_SCAN_REPORT", dict())
            .get("GLOSSARY", dict())
            .get("QID_LIST", dict())
            .get("QID", list())
        )
        glossary = {g["QID"]: g for g in glossary}
        for vulnerability in vulnerabilities:
            vulnerability.update(glossary.get(vulnerability.get("QID", ""), {}))
        return vulnerabilities

    def _get_findings_scan(self, input) -> list:
        findings = list()
        ips = input.get("SCAN", dict()).get("IP", list())
        if not isinstance(ips, list):
            ips = [ips]
        for ip in ips:
            vulns = ip.get("VULNS", list())
            if not isinstance(vulns, list):
                vulns = [vulns]
            for vuln in vulns:
                cats = vuln.get("CAT", list())
                if not isinstance(cats, list):
                    cats = [cats]
                for cat in cats:
                    finding = cat.get("VULN", dict())
                    # Remove special characters from field names
                    finding["CVEID"] = finding.pop("@cveid", "")

                    # Rename fields to match WAS format
                    finding["SEVERITY"] = finding.pop("@severity", "1")
                    finding["QID"] = finding.pop("@number", "")
                    finding["IMPACT"] = finding.pop("DIAGNOSIS", "")
                    finding["DESCRIPTION"] = finding.pop("CONSEQUENCE", "")
                    
                    # Add additional fields
                    finding["IP"] = ip.get("@value", "")

                    # Append to findings
                    findings.append(finding)
        return findings     

    def _get_findings(self, input) -> list:
        if "WAS_SCAN_REPORT" in input:
            return self._get_findings_was(input)
        elif "SCAN" in input:
            return self._get_findings_scan(input)
        else:
            raise NotImplementedError("This report type is not supported yet. Please open an issue at https://github.com/Syslifters/reptor/issues and provide a sample report.")

    def parse(self):
        super().parse()
        if not self.parsed_input:
            return
        if isinstance(self.parsed_input, list):
            # Multiple input files. Merge findings
            findings = []
            for input_file in self.parsed_input:
                findings.extend(self._get_findings(input_file))
        else:
            findings = self._get_findings(self.parsed_input)

        # Filter severity and plugins
        filtered = list()
        for finding in findings:
            for score_threshold, severity_label in self.score_mapping.items():
                if int(finding["SEVERITY"]) <= score_threshold:
                    finding["severity_label"] = severity_label
                    if severity_label in self.severity_filter:
                        filtered.append(finding)
                    break
        findings = filtered

        if self.included_plugins:
            findings = [ f for f in findings if f.get("QID", "") in self.included_plugins ]
        if self.excluded_plugins:
            findings = [ f for f in findings if f.get("QID", "") not in self.excluded_plugins ]

        # Sort
        findings = sorted(
            findings,
            key=lambda x: (int(x.get("SEVERITY", 0)), x.get("name", "")),
            reverse=True,
        )
        self.parsed_input = findings

    def aggregate_by_target(self) -> list:
        targets = dict()
        for finding in self.parsed_input:
            url = finding.get("IP") or finding.get("URL", "")
            finding["target"] = urlparse(url).netloc or finding.get("IP") or finding.get("URL") or "Unknown"
            targets.setdefault(finding["target"], list()).append(finding)

        # Sort hosts by highest severity
        targets = list(targets.values())
        targets.sort(
            key=lambda x: (int(x[0].get("SEVERITY", 0)), x[0].get("name", "")), reverse=True
        )
        return targets

    def aggregate_by_plugin(self) -> list:
        plugins = dict()
        for finding in self.parsed_input:
            plugins.setdefault(finding.get("QID", ""), list()).append(finding)

        # Sort by highest severity
        plugins = list(plugins.values())
        plugins.sort(
            key=lambda x: (int(x[0].get("severity", 0)), x[0].get("name", "")), reverse=True
        )
        return plugins

    @classmethod
    def merge_findings(cls, findings: list) -> dict:
        if not findings:
            return dict()
        m = findings[0].copy()
        for finding in findings:
            if url := finding.get('URL'):
                m.setdefault("URLS", list()).append(url)
            if param := finding.get('PARAM'):
                m.setdefault("PARAMS", list()).append(param)
            if access_path := finding.get('ACCESS_PATH'):
                m.setdefault("ACCESS_PATHS", list()).append(access_path)
            if ip := finding.get('IP'):
                m.setdefault("IPS", list()).append(ip)

        m["affected_components"] = sorted(set(m.get("URLS") or m.get("IPS") or []))
        return m

    def preprocess_for_template(self) -> list:
        plugin_findings = self.aggregate_by_plugin()
        findings = list()
        for plugin in plugin_findings:
            merged = self.merge_findings(plugin)
            merged["finding_templates"] = merged["QID"]
            findings.append(merged)
        return findings

    def finding_global(self):
        return self.preprocess_for_template()


loader = Qualys



import io
import json
import os
import pathlib
import subprocess
import tarfile

import pytest
import yaml
from reptor.plugins.core.Conf.tests.conftest import templates_api  # noqa: F401


class TestIntegrationTemplate(object):
    @pytest.mark.integration
    @pytest.mark.parametrize(
        "input_file",
        ["template_1.json", "template_2.json", "template_1.toml", "template_2.toml"],
    )
    def test_upload_finding_template(self, input_file, templates_api):  # noqa: F811
        # Delete the templates if already exist
        for title in [
            "SQL Injection (SQLi)",
            "Session management weaknesses",
            "My Title",
        ]:
            for id in set([t.id for t in templates_api.search(title)]):
                templates_api.delete_template(id)

        input_path = pathlib.Path(os.path.dirname(__file__)) / f"data/{input_file}"
        p = subprocess.Popen(
            ["reptor", "template", "--upload"],
            stderr=subprocess.PIPE,
            stdin=subprocess.PIPE,
        )
        _, output = p.communicate(input=input_path.read_bytes())
        assert p.returncode == 0
        output_lines = output.decode().splitlines()
        new_ids = [o.split()[-1] for o in output_lines[0:-1] if "Uploaded" in o]
        for new_id in new_ids:
            templates_api.delete_template(new_id)
        assert "Successfully uploaded" in output_lines[-1]

    def test_template_export_archive(self):
        p = subprocess.Popen(
            ["reptor", "template", "--search", "XE", "--export", "json"],
            stdout=subprocess.PIPE,
        )
        templates, _ = p.communicate()
        templates = json.loads(templates.decode())
        assert p.returncode == 0
        ids = [template["id"] for template in templates]

        p = subprocess.Popen(
            [
                "reptor",
                "template",
                "--search",
                "XE",
                "--export",
                "tar.gz",
                "--output",
                "-",
            ],
            stdout=subprocess.PIPE,
        )
        archive, _ = p.communicate()
        archive_io = io.BytesIO(archive)
        with tarfile.open(fileobj=archive_io, mode="r:gz") as tar:
            assert all(f"{id}.json" in list(tar.getnames()) for id in ids)
            assert len(tar.getmembers()) == len(ids)

    def test_template_export(self):
        input_path = pathlib.Path(os.path.dirname(__file__)) / "data/template_1.json"
        p = subprocess.Popen(
            ["reptor", "template", "--upload"],
            stderr=subprocess.PIPE,
            stdin=subprocess.PIPE,
        )
        _, output = p.communicate(input=input_path.read_bytes())
        assert p.returncode == 0

        p = subprocess.Popen(
            ["reptor", "template", "--list", "--export", "json"],
            stdout=subprocess.PIPE,
        )
        templates, _ = p.communicate()
        templates = json.loads(templates.decode())
        assert p.returncode == 0
        assert len(templates) > 0

        p = subprocess.Popen(
            ["reptor", "template", "--search", "SQL", "--export", "json"],
            stdout=subprocess.PIPE,
        )
        sql_templates, _ = p.communicate()
        sql_templates = json.loads(sql_templates.decode())
        assert p.returncode == 0
        assert len(templates) > len(sql_templates) > 0
        sql_summary = sql_templates[0]["translations"][0]["data"]["summary"]

        p = subprocess.Popen(
            ["reptor", "template", "--search", "SQL", "--export", "plain"],
            stdout=subprocess.PIPE,
        )
        sql_templates, _ = p.communicate()
        sql_templates = sql_templates.decode()
        assert p.returncode == 0
        assert sql_summary in sql_templates

        p = subprocess.Popen(
            ["reptor", "template", "--search", "SQL", "--export", "yaml"],
            stdout=subprocess.PIPE,
        )
        sql_templates, _ = p.communicate()
        sql_templates = yaml.safe_load(sql_templates.decode())
        assert p.returncode == 0
        assert len(templates) > len(sql_templates) > 0



import typing
from posixpath import join as urljoin

from reptor.api.APIClient import APIClient
from reptor.models.FindingTemplate import FindingTemplate


class TemplatesAPI(APIClient):
    def __init__(self, **kwargs) -> None:
        super().__init__(**kwargs)

        self.base_endpoint = urljoin(
            self.reptor.get_config().get_server(), "api/v1/findingtemplates/"
        )

    def get_template(self, template_id: str) -> FindingTemplate:
        """Gets a single Template by ID"""
        response = self.get(urljoin(self.base_endpoint, template_id))
        return FindingTemplate(response.json())

    def export(self, template_id: str) -> bytes:
        """Exports a template in archive format (tar.gz)"""
        url = urljoin(self.base_endpoint, f"{template_id}/export/")
        return self.post(url).content

    def search(self, search_term: str = "") -> typing.List[FindingTemplate]:
        """Searches through the templates"""
        templates_raw = self.get_paginated(self.base_endpoint, params={"search": search_term})
        return [FindingTemplate(template_raw) for template_raw in templates_raw]

    def get_templates_by_tag(self, tag: str) -> typing.List[FindingTemplate]:
        matched_templates = list()
        for finding_template in self.search(tag):
            if tag in finding_template.tags:
                matched_templates.append(finding_template)
        return matched_templates

    def upload_template(
        self, template: FindingTemplate
    ) -> typing.Optional[FindingTemplate]:
        """Uploads a new Finding Template to API

        Args:
            template (FindingTemplate): Model Data to upload

        Returns:
            FindingTemplate: Updated Model with ID etc.
        """
        existing_templates = self.search(template.translations[0].data.title)
        if any([t.translations[0].data.title == template.translations[0].data.title for t in existing_templates]):
            self.display(f"Template with title {template.translations[0].data.title} exists. Skipping.")
            return None
        
        res = self.post(
            self.base_endpoint,
            json=template.to_dict(),
        )
        return FindingTemplate(res.json())


    def delete_template(self, template_id: str) -> None:
        """Deletes a Template by ID"""
        self.delete(urljoin(self.base_endpoint, template_id))
        return



import os
from pathlib import Path

import pytest

from reptor.lib.plugins.TestCaseToolPlugin import TestCaseToolPlugin
from reptor.models.Note import NoteTemplate

from ..Qualys import Qualys


class TestQualys(TestCaseToolPlugin):
    templates_path = os.path.normpath(Path(os.path.dirname(__file__)) / "../templates")

    @pytest.fixture(autouse=True)
    def setUp(self) -> None:
        Qualys.setup_class(
            Path(os.path.dirname(self.templates_path)), skip_user_plugins=True
        )
        self.qualys = Qualys()

    def _load_xml_data(self, filename):
        self.qualys.input_format = "xml"
        filepath = os.path.join(os.path.dirname(__file__), f"./data/{filename}.xml")
        with open(filepath, "r") as f:
            self.qualys.raw_input = f.read()

    def test_parse(self):
        # WAS
        self._load_xml_data("webapp_scan")
        self.qualys.parse()
        assert len(self.qualys.parsed_input) == 18

        self.qualys.raw_input = [self.qualys.raw_input, self.qualys.raw_input]
        self.qualys.parse()
        assert len(self.qualys.parsed_input) == 36

        # VULN
        self._load_xml_data("vuln_scan")
        self.qualys.parse()
        assert len(self.qualys.parsed_input) == 2

    def test_parse_with_severity_filter(self):
        # WAS
        self._load_xml_data("webapp_scan")
        self.qualys.severity_filter = {"critical"}
        self.qualys.parse()
        p = self.qualys.parsed_input
        assert isinstance(p, list)
        assert len(p) == 7

        # VULN
        self._load_xml_data("vuln_scan")
        self.qualys.severity_filter = {"critical"}
        self.qualys.parse()
        p = self.qualys.parsed_input
        assert isinstance(p, list)
        assert len(p) == 0

    def test_parse_with_qid_filter(self):
        # WAS
        self._load_xml_data("webapp_scan")
        self.qualys.included_plugins = {"150158"}
        self.qualys.parse()
        p = self.qualys.parsed_input
        assert isinstance(p, list)
        assert all(f["QID"] == "150158" for f in p)

        self.qualys.included_plugins = {}
        self.qualys.excluded_plugins = {"150158"}
        self.qualys.parse()
        p = self.qualys.parsed_input
        assert isinstance(p, list)
        assert not any(f["QID"] == "150158" for f in p)

        # VULN
        self._load_xml_data("vuln_scan")
        self.qualys.included_plugins = {"86247"}
        self.qualys.parse()
        p = self.qualys.parsed_input
        assert isinstance(p, list)
        assert all(f["QID"] == "86247" for f in p)

        self.qualys.included_plugins = {}
        self.qualys.excluded_plugins = {"86247"}
        self.qualys.parse()
        p = self.qualys.parsed_input
        assert isinstance(p, list)
        assert not any(f["QID"] == "86247" for f in p)

    def test_preprocess_for_template(self):
        # WAS
        self._load_xml_data("webapp_scan")
        self.qualys.parse()
        p = self.qualys.preprocess_for_template()
        assert len(p) == 13
        assert isinstance(p[0]["affected_components"], list)
        assert isinstance(p[0]["URLS"], list)
        assert isinstance(p[0]["PARAMS"], list)
        assert isinstance(p[0]["ACCESS_PATHS"], list)

        # VULN
        self._load_xml_data("vuln_scan")
        self.qualys.parse()
        p = self.qualys.preprocess_for_template()
        assert len(p) == 2
        assert isinstance(p[0]["affected_components"], list)
        assert isinstance(p[0]["IPS"], list)

    def test_aggregate_by_plugin(self):
        self._load_xml_data("webapp_scan")
        self.qualys.parse()
        a = self.qualys.aggregate_by_plugin()
        assert len(a) == 13
        assert sum(len(v) for v in a) == 18
        
    def test_create_notes(self):
        # WAS
        self._load_xml_data("webapp_scan")
        self.qualys.parse()
        note = self.qualys.create_notes()
        assert isinstance(note, NoteTemplate)
        assert note.icon_emoji == "🛡️"
        assert note.title == "Qualys"
        assert note.children[0].title == "ginandjuice.shop"
        assert len(note.children[0].children) == 18

        # VULN
        self._load_xml_data("vuln_scan")
        self.qualys.parse()
        note = self.qualys.create_notes()
        assert isinstance(note, NoteTemplate)
        assert note.icon_emoji == "🛡️"
        assert note.title == "Qualys"
        assert note.children[0].title == "34.249.203.140"
        assert len(note.children[0].children) == 2

    def test_aggregate_by_target(self):
        self._load_xml_data("webapp_scan")
        self.qualys.parse()
        a = self.qualys.aggregate_by_target()
        assert len(a) == 1
        assert len(a[0]) == 18



import typing
from datetime import datetime
from os.path import basename
from posixpath import join as urljoin

from requests import HTTPError

from reptor.api.APIClient import APIClient
from reptor.models.Note import Note
from reptor.utils.file_operations import guess_filetype

from reptor.models.Note import NoteTemplate


class NotesAPI(APIClient):
    """Interacts with Notes Endpoints

    Args:
        APIClient (_type_): _description_
    """

    def __init__(self, **kwargs) -> None:
        super().__init__(require_project_id=False, **kwargs)

        if self.private_note:
            self.base_endpoint = urljoin(
                self.reptor.get_config().get_server(),
                "api/v1/pentestusers/self/notes/",
            )
        elif self.project_id:
            self.base_endpoint = urljoin(
                self.reptor.get_config().get_server(),
                f"api/v1/pentestprojects/{self.project_id}/notes/",
            )

    @property
    def private_note(self):
        return self.reptor.get_config().get("private_note")

    def get_notes(self) -> typing.List[Note]:
        """
        Gets list of notes
        (Notes are not paginated)
        """
        self.debug("Getting Notes List")
        response = self.get(self.base_endpoint)
        notes = list()
        for note_data in response.json():
            notes.append(Note(note_data))
        return notes

    def get_note(
        self,
        noteid: typing.Optional[str] = None,
        notetitle: typing.Optional[str] = None,
    ) -> typing.Optional[Note]:
        for note in self.get_notes():
            if noteid:
                if note.id == noteid:
                    return note
            elif notetitle:
                if note.title == notetitle:
                    return note
            else:
                raise ValueError("Either noteid or notetitle must be provided")

    def create_note(
        self,
        title="CLI Note",
        parent_id: typing.Optional[str] = None,
        order=None,
        checked=None,
        icon=None,
    ) -> Note:
        self.debug("Creating Note")
        if title is None:
            raise ValueError("Note title must not be null.")
        note = self.post(
            self.base_endpoint,
            json={
                "order": order,
                "parent": parent_id or None,
                "checked": checked,
                "title": title,
            },
        ).json()
        self.debug(f"We created note with {note}")
        if icon:
            self.set_icon(note.get("id"), icon)
        elif title == "Uploads":
            self.set_icon(note.get("id"), "📤")
        return Note(note)

    def delete_note(self, notes_id: str):
        url = urljoin(self.base_endpoint, f"{notes_id}/")
        self.delete(url)

    def set_icon(self, notes_id: str, icon: str):
        url = urljoin(self.base_endpoint, f"{notes_id}/")
        self.put(url, json={"icon_emoji": icon})

    def _upload_note(
        self,
        note: Note,
    ):
        self.debug(f"Note has length: {len(note.text)}")
        url = urljoin(self.base_endpoint, note.id, "")
        r = self.put(url, json=note.to_dict())

        try:
            r.raise_for_status()
            self.display(f'Note written to "{note.title}".')
        except HTTPError as e:
            raise HTTPError(
                f'{str(e)} Are you uploading binary content to note? (Try "file" subcommand)'
            ) from e

    def write_note_templates(
        self,
        note_templates: typing.Union[NoteTemplate, typing.List[NoteTemplate]],
        timestamp: bool = True,
        **kwargs,
    ):
        if not isinstance(note_templates, list):
            note_templates = [note_templates]
        for note_template in note_templates:
            if note_template.parent_notetitle and not note_template.parent:
                note_template.parent = self.get_or_create_note_by_title(
                    note_template.parent_notetitle
                ).id

            note = None
            if not note_template.force_new:
                note = self.get_note_by_title(
                    note_template.title,
                    parent_noteid=note_template.parent,
                )
            if note is None:
                new_note = True
                note = self.create_note(
                    title=note_template.title,
                    parent_id=note_template.parent or None,
                )
            else:
                new_note = False
            self.debug(f"Got note from server {note.title} ({note.id})")

            # Prepare note (append content to existing note, etc)
            note_text = note.text + "\n\n" if note.text else ""
            if (
                timestamp and note_template.text
            ):  # Only add timestamp if there is content
                note_text += f"[{datetime.now().strftime('%Y-%m-%d %H:%M:%S')}]"
                if "\n" in note_template.text:
                    if not note_template.text.startswith("\n"):
                        note_text += "\n"
                else:
                    note_text += ": "
            note_text += note_template.text or ""

            if new_note:
                upload_note = Note.from_note_template(note_template)
                upload_note.parent = note.parent
                upload_note.id = note.id
                upload_note.text = note_text
            else:
                upload_note = note
                upload_note.text = note_text

            # Upload note and children recursively
            self._upload_note(upload_note, **kwargs)
            for child in note_template.children:
                child.parent = note.id
                self.write_note_templates(child, timestamp=timestamp, **kwargs)

    def write_note(
        self,
        timestamp: bool = False,
        **kwargs,
    ):
        note_template = NoteTemplate.from_kwargs(**kwargs)
        self.write_note_templates(
            note_template, timestamp=timestamp
        )

    def get_note_by_title(
        self,
        title,
        parent_noteid=None,  # Preferred over parent_notetitle
        parent_notetitle=None,
        ignore_parent=False,
    ) -> typing.Optional[Note]:
        if not parent_noteid and parent_notetitle:
            try:
                parent_noteid = self.get_note_by_title(parent_notetitle, ignore_parent=True).id  # type: ignore
            except AttributeError:
                raise ValueError(f'Parent note "{parent_notetitle}" does not exist.')
        notes_list = self.get_notes()

        for note in notes_list:
            if note.title == title and (note.parent == parent_noteid or ignore_parent):
                break
        else:
            return None
        return note

    def get_or_create_note_by_title(
        self,
        title,
        parent_noteid=None,  # Preferred over parent_notetitle
        parent_notetitle=None,
        icon=None,
    ) -> Note:
        if not parent_noteid and parent_notetitle:
            parent_noteid = self.get_or_create_note_by_title(
                parent_notetitle, icon=icon
            ).id
        note = self.get_note_by_title(title, parent_noteid=parent_noteid)
        if not note:
            # Note does not exist. Create.
            note = self.create_note(title=title, parent_id=parent_noteid, icon=icon)
        return note

    def upload_file(
        self,
        file: typing.Optional[typing.IO] = None,
        content: typing.Optional[bytes] = None,
        notetitle: typing.Optional[str] = None,
        filename: typing.Optional[str] = None,
        caption: typing.Optional[str] = None,
        parent_notetitle: typing.Optional[str] = None,
        **kwargs,
    ):
        assert file or content
        assert not (file and content)
        if notetitle is None:
            notetitle = "Uploads"

        if file:
            if file.name == "<stdin>":
                self.display("Reading from stdin...")
            elif not filename:
                filename = basename(file.name)
            # TODO this might be streamed to not load entire file to memory
            try:
                content = file.buffer.read()  # type: ignore
            except AttributeError:
                content = file.read()
            if not filename:
                filetype = guess_filetype(content) or "dat"
                filename = f"data.{filetype}"

            if not content:
                self.warning(f"{file.name} is empty. Will not upload.")
                return

        self.get_or_create_note_by_title(
            notetitle, parent_notetitle=parent_notetitle
        )
        if self.private_note:
            url = urljoin(self.base_endpoint, "upload/")
        else:
            url = urljoin(self.base_endpoint.rsplit("/", 2)[0], "upload/")
        response_json = self.post(
            url, files={"file": (filename, content)}, json_content=False
        ).json()
        is_image = True if response_json.get("resource_type") == "image" else False
        if is_image:
            file_path = f"/images/name/{response_json['name']}"
            note_content = f"\n![{caption or filename}]({file_path})"
        else:
            file_path = f"/files/name/{response_json['name']}"
            note_content = f"\n[{caption or filename}]({file_path})"

        self.write_note(
            title=notetitle,
            text=note_content,
            parent_notetitle=parent_notetitle,
            **kwargs,
        )



import io
import json
import sys
from io import StringIO
from unittest.mock import MagicMock

import pytest
import tomli_w

from reptor.lib.reptor import reptor
from reptor.models.ProjectDesign import ProjectDesignField
from reptor.models.Section import Section

from ..PushProject import PushProject


class TestPushProject:
    valid_data = {
        "report_data": {
            "title": "NEW REPORT",
            "custom_report_field": "FIELD VALUE",
        },
        "findings": [
            {
                "data": {
                    "title": "123",
                    "custom_finding_field": "FINDING FIELD VALUE",
                }
            }
        ],
    }

    @pytest.fixture(autouse=True)
    def setup(self):
        self.reptor = reptor

    def get_mocked_push_project(self, projectdata):
        pp = PushProject(projectdata=io.BytesIO(projectdata.encode()))

        pp.reptor.api.projects.project = MagicMock()
        pp.reptor.api.projects.project.name = "my-project"
        pp.reptor.api.projects.project.sections = [
            Section({"id": "section1", "fields": ["title"]}),
            Section({"id": "section2", "fields": ["custom_report_field"]}),
        ]
        self.reptor.api.projects._project_dict = {
            "id": "db837c68-ff58-4f63-9161-d2310d71999b",
            "project_type": "c357c387-baff-42ce-8e79-eb0597c3e0e8",
        }
        pp.reptor.api.project_designs.project_design = MagicMock()
        pp.reptor.api.project_designs.project_design.report_fields = [
            ProjectDesignField({"name": "title", "type": "string"}),
            ProjectDesignField({"name": "custom_report_field", "type": "string"}),
        ]
        pp.reptor.api.project_designs.project_design.finding_fields = [
            ProjectDesignField({"name": "title", "type": "string"}),
            ProjectDesignField({"name": "custom_finding_field", "type": "string"}),
        ]
        self.reptor.api.projects._update_section = MagicMock()
        self.reptor.api.projects.create_finding = MagicMock()
        return pp

    def test_read_valid_input(self):
        stdin = sys.stdin
        try:
            for dumps in [json.dumps, tomli_w.dumps]:
                sys.stdin = StringIO(dumps(self.valid_data))
                pp = PushProject()
                assert isinstance(pp.projectdata, dict)
                assert pp.projectdata["report_data"]["title"] == "NEW REPORT"
                assert pp.projectdata["report_data"]["custom_report_field"] == "FIELD VALUE"
                assert pp.projectdata["findings"][0]["data"]["title"] == "123"
                assert (
                    pp.projectdata["findings"][0]["data"]["custom_finding_field"]
                    == "FINDING FIELD VALUE"
                )
        finally:
            sys.stdin = stdin

    def test_upload_valid_data(self):
        for dumps in [json.dumps, tomli_w.dumps]:
            pp = self.get_mocked_push_project(dumps(self.valid_data))
            pp.run()

            assert self.reptor.api.projects._update_section.call_count == 2
            assert self.reptor.api.projects.create_finding.call_count == 1

    @pytest.mark.parametrize(
        "data",
        [
            {"report_data": {"title": 123}},
            {"report_data": {"custom_report_field": 123}},
            {"findings": [{"data": {"title": 123}}]},
            {"findings": [{"data": {"custom_finding_field": 123}}]},
        ],
    )
    def test_errors(self, data):
        with pytest.raises(ValueError):
            pp = self.get_mocked_push_project(tomli_w.dumps(data))
            pp.run()



import json
import os
from unittest.mock import MagicMock

import pytest
import requests
from requests.exceptions import HTTPError

from reptor.lib.conf import Config
from reptor.lib.reptor import reptor

from .. import DefectDojo


class TestDefectDojo:
    class MockResponse:
        def __init__(self, content, status_code, raise_exception=False):
            self.content = content
            self.status_code = status_code
            self.raise_exception = raise_exception

        def raise_for_status(self):
            if self.raise_exception:
                raise HTTPError("Mocked HTTPError")
            return

        def json(self):
            return self.content

    @pytest.fixture(autouse=True)
    def setUp(self):
        self.reptor = reptor
        self.reptor._config = Config()
        self.reptor._config._raw_config = {}

    def test_init(self):
        self.reptor.get_config()
        with pytest.raises(ValueError):
            # Should raise because of missing URL
            DefectDojo.DefectDojo()

        with pytest.raises(ValueError):
            # Should raise because of missing API key
            DefectDojo.DefectDojo(url="test")

    def test_run(self):
        finding_data = json.loads(
            open(
                os.path.join(os.path.dirname(__file__), "./data/finding_data.json")
            ).read()
        )
        url = "http://localhost:8080"

        # Test Defect Dojo API key
        dd_apikey = "ef8d9jd996b8p17534383cbte26red188d59c4bs"
        DefectDojo.DefectDojo.apikey = dd_apikey

        requests.get = MagicMock(return_value=self.MockResponse(finding_data, 200))
        defectdojo = DefectDojo.DefectDojo(url=url)
        defectdojo._upload_finding_template = MagicMock()

        defectdojo.run()

        defectdojo._upload_finding_template.assert_called_once()
        uploaded_finding = defectdojo._upload_finding_template.call_args.args[0]
        assert (
            uploaded_finding.translations[0].data.title
            == "AVD-AWS-0107 - An Ingress Security Group Rule Allows Traffic From /0."
        )

    def test_get_findings(self):
        finding_data = json.loads(
            open(
                os.path.join(os.path.dirname(__file__), "./data/finding_data.json")
            ).read()
        )
        url = "http://localhost:8080"

        # Test Defect Dojo API key
        dd_apikey = "ef8d9jd996b8p17534383cbte26red188d59c4bs"
        DefectDojo.DefectDojo.apikey = dd_apikey

        requests.get = MagicMock(return_value=self.MockResponse(finding_data, 200))
        defectdojo = DefectDojo.DefectDojo(url=url)

        assert finding_data == defectdojo._get_defectdojo_findings()



import json
import subprocess
import time

import pytest

from reptor.plugins.core.Conf.tests.conftest import projects_api  # noqa: F401


@pytest.mark.integration
class TestIntegrationFinding(object):
    def test_push_valid_finding(self, projects_api):  # noqa: F811
        title = str(time.time())
        reference = "https://example.com/" + title
        affected_component = "https://example.com/affected/" + title
        finding = {
            "status": "in-progress",
            "data": {
                "cvss": "CVSS:3.1/AV:N/AC:L/PR:H/UI:N/S:U/C:L/I:L/A:N",
                "title": title,
                "inexistent-field": "should be ignored",
                "summary": "We detected a reflected XSS vülnerability.",
                "references": [reference],
                "description": "The impact was heavy.",
                "recommendation": "HTML encode user-supplied inputs.",
                "affected_components": [
                    affected_component,
                    "https://example.com/q=alert(1)",
                ],
            },
        }
        p = subprocess.Popen(
            ["reptor", "finding"],
            stdout=subprocess.PIPE,
            stdin=subprocess.PIPE,
        )
        p.communicate(input=json.dumps(finding).encode())
        assert p.returncode == 0

        findings = projects_api.get_findings()
        f = None
        for f in findings:
            if f.data.title == title:
                break

        assert f is not None
        assert f.data.title == title
        assert f.data.references == [reference]
        assert f.data.affected_components == [
            affected_component,
            "https://example.com/q=alert(1)",
        ]
        assert f.data.summary == "We detected a reflected XSS vülnerability."

    def test_push_invalid_finding(self, projects_api):  # noqa: F811
        title = str(time.time())
        finding = {
            "status": "in-progress",
            "data": {
                "cvss": "CVSS:3.1/AV:N/AC:L/PR:H/UI:N/S:U/C:L/I:L/A:N",
                "title": title,
                "inexistent-field": "should be ignored",
                "summary": "We detected a reflected XSS vulnerability.",
                "references": [],
                "description": "The impact was heavy.",
                "recommendation": "HTML encode user-supplied inputs.",
                "affected_components": "ERROR: This should be list",
            },
        }
        p = subprocess.Popen(
            ["reptor", "finding"],
            stdout=subprocess.PIPE,
            stderr=subprocess.PIPE,
            stdin=subprocess.PIPE,
        )
        _, err = p.communicate(input=json.dumps(finding).encode())
        assert p.returncode == 2
        findings = projects_api.get_findings()
        assert title not in [f.data.title for f in findings]
        assert (
            "affected_components" in err.decode()
        )  # The errornous field should occur in error message



import argparse
import typing

from reptor.lib.plugins.ToolBase import ToolBase
from reptor.models.Note import NoteTemplate
from reptor.models.UserConfig import UserConfig


class OpenVAS(ToolBase):
    meta = {
        "author": "Syslifters",
        "name": "OpenVAS",
        "version": "1.0",
        "license": "MIT",
        "tags": [],
        "summary": "OpenVAS vulnerability importer",
    }

    score_mapping = {
        0: "info",
        3.9: "low",
        6.9: "medium",
        8.9: "high",
        10: "critical",
    }
    severity_mapping = {
        "info": "🟢",
        "low": "🔵",
        "medium": "🟡",
        "high": "🟠",
        "critical": "🔴",
    }
    supports_multi_input = True

    @property
    def user_config(self) -> typing.List[UserConfig]:
        return [
            UserConfig(
                name="severity_filter",
                friendly_name='Severity filter (e.g., "medium-critical")',
                callback=self._parse_severity_filter,
            ),
            UserConfig(
                name="min_qod",
                friendly_name="Minimum QoD (0-100)",
                callback=int,
            ),
            UserConfig(
                name="excluded_plugins",
                friendly_name="Exclude plugin IDs (comma-separated)",
                callback=UserConfig.split,
            ),
            UserConfig(
                name="included_plugins",
                friendly_name="Include plugin IDs (comma-separated)",
                callback=UserConfig.split,
            ),
        ]

    def __init__(self, **kwargs):
        super().__init__(**kwargs)
        self.notetitle = kwargs.get("notetitle") or "OpenVAS"
        self.note_icon = "🦖"
        self.input_format = "xml"
        self.timestamp = False
        self.severity_filter = getattr(
            self, "severity_filter", None
        ) or self._parse_severity_filter(kwargs.get("severity_filter", "low-critical"))
        self.min_qod = getattr(self, "min_qod", None) or kwargs.get("min_qod", 0)

        self.included_plugins = getattr(self, "included_plugins", list())
        self.included_plugins += list(
            filter(None, ((kwargs.get("included_plugins")) or "").split(","))
        )
        self.excluded_plugins = getattr(self, "excluded_plugins", list())
        self.excluded_plugins += list(
            filter(None, ((kwargs.get("excluded_plugins")) or "").split(","))
        )

    def _parse_severity_filter(self, filter: str) -> set:
        if filter is None:
            return set(self.severity_mapping.keys())
        filter_elements = set()
        if "-" in filter:
            filter_elements = filter.split("-")
            try:
                filter_indexes = [
                    list(self.severity_mapping.keys()).index(f) for f in filter_elements
                ]
            except ValueError:
                raise ValueError(
                    "Invalid filter range. Use keywords from 'info,low,medium,high,critical'"
                )
            filter_elements = list()
            for i in range(min(filter_indexes), max(filter_indexes) + 1):
                filter_elements += [list(self.severity_mapping.keys())[i]]
        elif filter:
            filter_elements = filter.split(",")
            if not all(f in self.severity_mapping for f in filter_elements):
                raise ValueError(
                    "Invalid filter. Use keywords from 'info,low,medium,high,critical'"
                )
        if filter_elements:
            return set(filter_elements)
        else:
            return set(self.severity_mapping.keys())

    @classmethod
    def add_arguments(cls, parser, plugin_filepath=None):
        super().add_arguments(parser, plugin_filepath=plugin_filepath)
        parser.add_argument(
            "--severity-filter",
            help='Filter findings by severity comma-separated ("high,medium") or as range ("medium-critical")',
            action="store",
            dest="severity_filter",
            default=None,
        )
        parser.add_argument(
            "--severityfilter",
            help=argparse.SUPPRESS,
            action="store",
            dest="severity_filter",
            default=None,
        )
        parser.add_argument(
            "--min-qod",
            help="Minimum OpenVAS Quality of Detection (QoD) to include (0-100)",
            action="store",
            type=int,
            dest="min_qod",
            default=0,
        )
        parser.add_argument(
            "--minqod",
            help=argparse.SUPPRESS,
            action="store",
            type=int,
            dest="min_qod",
            default=0,
        )
        parser.add_argument(
            "--exclude",
            help="Exclude plugin IDs, comma-separated",
            action="store",
            dest="excluded_plugins",
            default=None,
        )
        parser.add_argument(
            "--include",
            help="Include plugin IDs, comma-separated; default: all are included",
            action="store",
            dest="included_plugins",
            default=None,
        )

    def create_notes(self):
        self.parse()
        data = self.aggregate_by_target()
        # Create note structure
        ## Main note
        main_note = NoteTemplate()
        main_note.title = self.notetitle
        main_note.icon_emoji = self.note_icon

        ## Subnotes per target
        for findings in data:
            host_overview = NoteTemplate()
            host_overview.title = f"{findings[0]['target']}"
            host_overview.checked = False
            host_overview.template = "host-overview"
            host_overview.template_data = {"data": findings}
            for f in findings:
                finding_note = NoteTemplate()
                finding_note.title = f"{ self.severity_mapping.get(f.get('risk_factor', 'info').lower()) } {f['name']}"
                finding_note.checked = False
                finding_note.template = "finding"
                finding_note.template_data = f
                finding_note.force_new = True
                host_overview.children.append(finding_note)
            main_note.children.append(host_overview)
        if not main_note.children:
            return None
        return main_note

    def _get_findings(self, input) -> list:
        if "get_reports_response" in input:
            input = input["get_reports_response"]
        results = input.get("report", dict())
        if results.get("report_format", dict()).get("name") == "Anonymous XML":
            self.log.warning("Anonymous XML. You might want to export as XML instead.")
        return (
            results.get("report", dict()).get("results", dict()).get("result", list())
        )

    def parse(self):
        super().parse()
        if not self.parsed_input:
            return
        if isinstance(self.parsed_input, list):
            # Multiple input files. Merge findings
            findings = self._get_findings(self.parsed_input[0])
            for i in range(1, len(self.parsed_input)):
                findings += self._get_findings(self.parsed_input[i])
        else:
            findings = self._get_findings(self.parsed_input)

        # Filter severity and plugins
        filtered = list()
        for finding in findings:
            finding["severity"] = float(finding.get("severity", "0"))
            for score_threshold, risk_factor in self.score_mapping.items():
                if finding["severity"] <= score_threshold:
                    finding["risk_factor"] = risk_factor
                    try:
                        finding["nvt"]["oid"] = finding["nvt"].pop("@oid")
                    except (KeyError, AttributeError):
                        finding["nvt"]["oid"] = ""
                    if risk_factor in self.severity_filter:
                        filtered.append(finding)
                    break

        findings = [
            f
            for f in filtered
            if self.min_qod <= int(f.get("qod", dict()).get("value", "0"))
        ]
        if self.included_plugins:
            findings = [
                f
                for f in findings
                if f.get("nvt", dict()).get("oid") in self.included_plugins
            ]
        if self.excluded_plugins:
            findings = [
                f
                for f in findings
                if f.get("nvt", dict()).get("oid") not in self.excluded_plugins
            ]

        for f in findings:
            # Parse nvt tags
            tags = f.get("nvt", dict()).get("tags", "")
            tags = {i[0]: i[1] for i in (item.split("=") for item in tags.split("|"))}
            f["nvt"]["tags"] = tags

            # Get target name
            f["target"] = f.get("host", dict()).get("hostname") or f.get(
                "host", dict()
            ).get("#text", "")
            try:
                f["host"]["ip"] = f["host"].pop("#text")
            except (KeyError, AttributeError):
                f["host"]["ip"] = ""

        # Sort
        findings = sorted(
            findings,
            key=lambda x: (x.get("severity", 0), x.get("name", "")),
            reverse=True,
        )
        self.parsed_input = findings

    def aggregate_by_target(self) -> list:
        targets = dict()
        for finding in self.parsed_input:
            targets.setdefault(finding["target"], list()).append(finding)

        # Sort hosts by highest severity
        targets = list(targets.values())
        targets.sort(
            key=lambda x: (x[0].get("severity", 0), x[0].get("name", "")), reverse=True
        )
        return targets

    def aggregate_by_plugin(self) -> list:
        plugins = dict()
        for finding in self.parsed_input:
            plugins.setdefault(
                finding.get("nvt", dict()).get("oid", ""), list()
            ).append(finding)

        # Sort by highest severity
        plugins = list(plugins.values())
        plugins.sort(
            key=lambda x: (x[0].get("severity", 0), x[0].get("name", "")), reverse=True
        )
        return plugins

    @classmethod
    def merge_findings(cls, findings: list) -> dict:
        m = dict()
        if not findings:
            return m
        for finding in findings:
            for k, v in finding.get("nvt", dict()).get("tags", dict()).items():
                m.setdefault(k, v)
            if "cvss_base_vector" in m:
                m.setdefault("cvss_vector", cls.cvss2_to_3(m["cvss_base_vector"]))
            if "name" in finding.keys():
                m.setdefault("name", finding["name"])
            if "description" in finding.keys():
                m.setdefault("description", list()).append(finding["description"])
            if "oid" in finding.get("nvt", dict()).keys():
                m.setdefault("oid", finding["nvt"]["oid"])
            if "risk_factor" in finding.keys():
                m.setdefault("risk_factor", finding["risk_factor"].lower())
            if "refs" in finding.get("nvt", dict()).keys():
                m.setdefault("refs", list())
                refs = finding["nvt"]["refs"].get("ref", dict)
                if isinstance(refs, dict):
                    refs = [refs]
                for ref in refs:
                    m["refs"].append(ref.get("@id", ""))
                m["refs"] = sorted(set(m["refs"]))
            m.setdefault("affected_components", list()).append(
                f"{finding['target']}{':' + finding['port'] if not finding['port'].startswith('general') else ''}"
            )
        m["affected_components"] = sorted(set(m["affected_components"]))
        return m

    def preprocess_for_template(self) -> list:
        plugin_findings = self.aggregate_by_plugin()
        findings = list()
        for plugin in plugin_findings:
            merged = self.merge_findings(plugin)
            merged["severity"] = merged["risk_factor"]
            merged["finding_templates"] = merged["oid"]
            findings.append(merged)
        return findings

    def finding_global(self):
        return self.preprocess_for_template()


loader = OpenVAS



import json
import subprocess

import pytest
import tomli
import yaml

from reptor.plugins.core.Conf.tests.conftest import get_note, projects_api  # noqa: F401


@pytest.mark.integration
class TestIntegrationExportFinding(object):
    def test_export_findings(self, projects_api):  # noqa: F811
        p = subprocess.Popen(
            ["reptor", "exportfindings", "--format", "json", "--fieldnames" "'id,title'"],
            stdout=subprocess.PIPE,
            stderr=subprocess.PIPE,
        )
        out, _ = p.communicate()
        assert p.returncode == 0
        json_export = json.loads(out)

        p = subprocess.Popen(
            ["reptor", "exportfindings", "--format", "toml"],
            stdout=subprocess.PIPE,
            stderr=subprocess.PIPE,
        )
        out, _ = p.communicate()
        assert p.returncode == 0
        toml_export = tomli.loads(out.decode()).get("findings")  # TOML wraps in a dict

        p = subprocess.Popen(
            ["reptor", "exportfindings", "--format", "yaml"],
            stdout=subprocess.PIPE,
            stderr=subprocess.PIPE,
        )
        out, _ = p.communicate()
        assert p.returncode == 0
        yaml_export = yaml.safe_load(out.decode())

        assert json_export == toml_export == yaml_export

        p = subprocess.Popen(
            ["reptor", "exportfindings", "--format", "json", "--upload"],
            stdout=subprocess.PIPE,
            stderr=subprocess.PIPE,
        )
        out, _ = p.communicate()
        assert p.returncode == 0

        note = get_note("Uploads", None)
        note_last_line = note["text"].splitlines()[-1]  # type: ignore
        assert f"[{projects_api.project.name}.json]" in note_last_line

    @pytest.mark.parametrize(
        "score,severity,vector",
        [
            (9.2, "Critical", "CVSS:4.0/AV:N/AC:H/AT:P/PR:N/UI:N/VC:H/VI:H/VA:H/SC:N/SI:N/SA:N"),
            (9.2, "Critical", "CVSS:4.0/AV:N/AC:L/AT:N/PR:N/UI:N/VC:H/VI:H/VA:H/SC:N/SI:N/SA:N/CR:M/MAV:N/MAT:P"),
            (8.4, "High", "CVSS:4.0/AV:N/AC:H/AT:P/PR:N/UI:N/VC:L/VI:H/VA:H/SC:N/SI:N/SA:N"),
            (4.2, "Medium", "CVSS:3.1/AV:N/AC:H/PR:L/UI:N/S:U/C:L/I:L/A:L/E:U/RC:U"),
            (6.0, "Medium", "CVSS:3.1/AV:N/AC:H/PR:L/UI:N/S:U/C:H/I:H/A:L/E:U/RC:U"),
            (2.4, "Low", "CVSS:3.0/AV:P/AC:L/PR:N/UI:N/S:U/C:L/I:N/A:N"),
            (5.8, "Medium", "AV:L/AC:M/Au:M/C:N/I:C/A:C"),
            (0.0, "None", "CVSS:3.1/AV:N/AC:L/PR:N/UI:N/S:U/C:N/I:N/A:N"),
        ],
    )
    def test_cvss_version_exports(self, score, severity, vector):
        finding = {
            "status": "in-progress",
            "data": {
                "title": "Finding Title CVSS",
                "cvss": vector,
            },
        }
        p = subprocess.Popen(
            ["reptor", "finding"],
            stdout=subprocess.PIPE,
            stderr=subprocess.PIPE,
            stdin=subprocess.PIPE,
        )
        out, err = p.communicate(input=json.dumps(finding).encode())
        assert p.returncode == 0

        p = subprocess.Popen(
            ["reptor", "exportfindings", "--format", "json"],
            stdout=subprocess.PIPE,
            stderr=subprocess.PIPE,
        )
        out, _ = p.communicate()
        results = json.loads(out)
        for result in results:
            if result.get("title") == "Finding Title CVSS":
                assert result["cvss__score"] == str(score)
                assert result["cvss__severity"] == severity
                assert result["cvss__vector"] == vector
                break
        assert p.returncode == 0